import argparse
from Bio import SeqIO

merge_seq_usage = '''
===================== merge_seq example commands =====================

BioSAK merge_seq -1 ctgs1.fa -2 ctgs2.fa -o combined_ctgs.fa -sl
BioSAK merge_seq -1 reads1.fq -2 reads2.fq -o combined_reads.fq -fq

======================================================================
'''


def merge_seq(args):

    # read in argument
    seq_file_1  = args['1']
    seq_file_2  = args['2']
    output_file = args['o']
    in_fastq    = args['fq']
    one_line    = args['sl']

    file_format = 'fasta'
    if in_fastq is True:
        file_format = 'fastq'

    wrote_seq_set = set()
    output_file_handle = open(output_file, 'w')

    # read in seq_file_1
    for each_seq in SeqIO.parse(seq_file_1, file_format):
        if each_seq.id not in wrote_seq_set:
            if in_fastq is False:
                if one_line is False:
                    SeqIO.write(each_seq, output_file_handle, 'fasta')
                else:
                    SeqIO.write(each_seq, output_file_handle, 'fasta-2line')
            else:
                SeqIO.write(each_seq, output_file_handle, 'fastq')
            wrote_seq_set.add(each_seq.id)

    # read in seq_file_2
    for each_seq in SeqIO.parse(seq_file_2, file_format):
        if each_seq.id not in wrote_seq_set:
            if in_fastq is False:
                if one_line is False:
                    SeqIO.write(each_seq, output_file_handle, 'fasta')
                else:
                    SeqIO.write(each_seq, output_file_handle, 'fasta-2line')
            else:
                SeqIO.write(each_seq, output_file_handle, 'fastq')
            wrote_seq_set.add(each_seq.id)

    output_file_handle.close()


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=merge_seq_usage)
    parser.add_argument('-1',       required=True,                          help='input file 1')
    parser.add_argument('-2',       required=True,                          help='input file 2')
    parser.add_argument('-o',       required=True,                          help='output sequence file')
    parser.add_argument('-fq',      required=False, action="store_true",    help='in fastq format, default: fasta')
    parser.add_argument('-sl',      required=False, action="store_true",    help='write out sequence in single line format')
    args = vars(parser.parse_args())
    merge_seq(args)



import os
import glob
import argparse
from Bio import SeqIO
import multiprocessing as mp
from Bio.SeqRecord import SeqRecord
from Bio.Seq import Seq


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def gbk_to_ffn_faa(gbk_in, ffn_out, faa_out):

    _, _, gbk_base, _ = sep_path_basename_ext(gbk_in)
    faa_out_handle = open(faa_out, 'w')
    ffn_out_handle = open(ffn_out, 'w')
    for seq_record in SeqIO.parse(gbk_in, 'genbank'):
        record_sequence = str(seq_record.seq)
        for feature in seq_record.features:
            if feature.type == 'CDS':
                feature_pos = '%s-%s' % (feature.location.start, feature.location.end)
                feature_seq_nt = record_sequence[feature.location.start:feature.location.end]
                if feature.location.strand in ['-1', -1]:
                    feature_pos = '%s-%s' % (feature.location.end, feature.location.start)
                    feature_seq_nt = str(Seq(feature_seq_nt).reverse_complement())
                feature_seq_aa = ''
                if 'translation' in feature.qualifiers:
                    feature_seq_aa = feature.qualifiers['translation'][0]

                protein_id = ''
                if 'protein_id' in feature.qualifiers:
                    protein_id = feature.qualifiers['protein_id'][0]

                cds_id_to_used = feature_pos.replace('<', '').replace('>', '')
                if protein_id != '':
                    cds_id_to_used = protein_id

                feature_product = ''
                if 'protein_id' in feature.qualifiers:
                    feature_product = feature.qualifiers['product'][0]

                gene_name = ''
                if 'gene' in feature.qualifiers:
                    gene_name = feature.qualifiers['gene'][0]

                feature_note = ''
                if 'note' in feature.qualifiers:
                    feature_note = feature.qualifiers['note'][0]

                desc_to_use = ''
                if feature_product != '':
                    desc_to_use = feature_product
                elif feature_note != '':
                    desc_to_use = feature_note
                elif gene_name != '':
                    desc_to_use = gene_name

                # write out nt sequence
                ffn_out_handle.write('>%s__%s %s\n' % (gbk_base, cds_id_to_used, desc_to_use))
                ffn_out_handle.write('%s\n' % feature_seq_nt)

                # write out aa sequence
                if feature_seq_aa != '':
                    faa_out_handle.write('>%s__%s %s\n' % (gbk_base, cds_id_to_used, desc_to_use))
                    faa_out_handle.write('%s\n' % feature_seq_aa)
    faa_out_handle.close()
    ffn_out_handle.close()


file_dir = '/Users/songweizhi/Desktop/Sponge_r226/sponge_phylogeny/COI/RefSeq_COI_wd/tmp'
file_ext = 'gbk'
file_re = '%s/*.%s' % (file_dir, file_ext)
file_list = glob.glob(file_re)

for file in sorted(file_list):
    faa_file = file.replace('.gbk', '.faa')
    ffn_file = file.replace('.gbk', '.ffn')
    gbk_to_ffn_faa(file, ffn_file, faa_file)


# gbk_file = '/Users/songweizhi/Desktop/Sponge_r226/sponge_phylogeny/to_add/COI_wd/tmp/KR911863.1.gbk'
# fna_file = '/Users/songweizhi/Desktop/KR911863.1.fas'
# faa_file = '/Users/songweizhi/Desktop/KR911863.1.faa'
#
# gbk_to_ffn_faa(gbk_file, fna_file, faa_file)




import argparse


combine_fun_stats_usage = '''
========================================= combine_fun_stats example commands =========================================

BioSAK combine_fun_stats -f1 sponge_ko_c_stats.txt -f2 seawater_ko_c_stats.txt -l1 Sponge -l2 Seawater -o summary.txt

======================================================================================================================
'''

def combine_fun_stats(args):

    stats_1_txt     = args['f1']
    stats_1_label   = args['l1']
    stats_2_txt     = args['f2']
    stats_2_label   = args['l2']
    op_txt          = args['o']

    overall_fun_set = set()
    fun_desc_dict = dict()
    stats_1_dict = dict()
    stats_1_sum = 0
    for each_line in open(stats_1_txt):
        each_line_split = each_line.strip().split('\t')
        fun_id = each_line_split[0]
        fun_num = int(each_line_split[1])
        fun_desc = each_line_split[2]
        stats_1_dict[fun_id] = fun_num
        fun_desc_dict[fun_id] = fun_desc
        stats_1_sum += fun_num
        overall_fun_set.add(fun_id)

    stats_2_dict = dict()
    stats_2_sum = 0
    for each_line in open(stats_2_txt):
        each_line_split = each_line.strip().split('\t')
        fun_id = each_line_split[0]
        fun_num = int(each_line_split[1])
        fun_desc = each_line_split[2]
        stats_2_dict[fun_id] = fun_num
        fun_desc_dict[fun_id] = fun_desc
        stats_2_sum += fun_num
        overall_fun_set.add(fun_id)

    stats_1_dict_pct = dict()
    for i in stats_1_dict:
        i_pct = stats_1_dict[i]*100/stats_1_sum
        i_pct = float("{0:.2f}".format(i_pct))
        stats_1_dict_pct[i] = i_pct

    stats_2_dict_pct = dict()
    for i in stats_2_dict:
        i_pct = stats_2_dict[i]*100/stats_2_sum
        i_pct = float("{0:.2f}".format(i_pct))
        stats_2_dict_pct[i] = i_pct

    op_txt_handle = open(op_txt, 'w')
    op_txt_handle.write('\t%s\t%s\tRatio\n' % (stats_1_label, stats_2_label))
    for each_fun in sorted(list(overall_fun_set)):
        fun_desc = fun_desc_dict[each_fun]
        fun_pct_1 = stats_1_dict_pct.get(each_fun, 0)
        fun_pct_2 = stats_2_dict_pct.get(each_fun, 0)

        ratio = 9999
        if fun_pct_2 != 0:
            ratio = fun_pct_1/fun_pct_2
        ratio = float("{0:.2f}".format(ratio))

        op_txt_handle.write('%s__%s\t%s\t%s\t%s\n' % (each_fun, fun_desc, fun_pct_1, fun_pct_2, ratio))
    op_txt_handle.close()


if __name__ == '__main__':

    combine_fun_stats_parser = argparse.ArgumentParser(usage=combine_fun_stats_usage)
    combine_fun_stats_parser.add_argument('-f1',    required=True,  help='fun stats 1')
    combine_fun_stats_parser.add_argument('-f2',    required=True,  help='fun stats 2')
    combine_fun_stats_parser.add_argument('-l1',    required=True,  help='fun stats 1 label')
    combine_fun_stats_parser.add_argument('-l2',    required=True,  help='fun stats 2 label')
    combine_fun_stats_parser.add_argument('-o',     required=True,  help='output directory')
    args = vars(combine_fun_stats_parser.parse_args())
    combine_fun_stats(args)




def keep_top_n_blast_hit(file_in, file_out, top_hit_num):

    file_out_handle = open(file_out, 'w')

    current_query_id = ''
    current_query_wrote_hits = 0
    for blast_hit in open(file_in):

        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]

        if current_query_id == '':
            current_query_id = query_id
            file_out_handle.write(blast_hit)
            current_query_wrote_hits += 1

        elif current_query_id != '':
            if query_id == current_query_id:
                if current_query_wrote_hits < top_hit_num:
                    file_out_handle.write(blast_hit)
                    current_query_wrote_hits += 1
            else:
                current_query_id = query_id
                file_out_handle.write(blast_hit)
                current_query_wrote_hits = 1

    file_out_handle.close()


gtdb_ssu_file = '/Users/songweizhi/Desktop/ssu_all_r202.fna'
blast_op_best_hit = '/Users/songweizhi/Desktop/Pig_1-25_16S_uclust_0.999.QC_vs_GTDB_r202_blastn.tab'


ref_seq_to_taxon_dict = {}
for each_line in open(gtdb_ssu_file):
    if each_line.startswith('>'):

        each_line_split = each_line.strip()[1:].split(' ')
        ref_seq_id = each_line_split[0]

        # get ref_seq_des
        ref_seq_des = ' '.join(each_line_split[1:])
        if ' [' in ref_seq_des:
            ref_seq_des = ref_seq_des.split(' [')[0]
        if ' ' in ref_seq_des:
            ref_seq_des = '_'.join(ref_seq_des.split(' '))

        ref_seq_to_taxon_dict[ref_seq_id] = ref_seq_des

for each_hit in open(blast_op_best_hit):
    each_hit_split  = each_hit.strip().split('\t')
    query_id        = each_hit_split[0]
    ref_id          = each_hit_split[1]
    identity        = float(each_hit_split[2])
    aln_len         = int(each_hit_split[3])
    ref_taxon       = ref_seq_to_taxon_dict.get(ref_id, 'NA')
    if (query_id == 'Pig_subsample_25_5754') and ('Oligosphaerales' in ref_taxon):
    #if (query_id == 'Pig_subsample_25_5354'):
        print('%s\t%s\t%s\t%s\t%s' % (query_id, ref_id, identity, aln_len, ref_taxon))



#!/usr/bin/env python3

import os
import argparse
from datetime import datetime
from BioSAK.global_functions import time_format


top_16S_hits_usage = '''
======================================= top_16S_hits example commands =======================================

module load blast+/2.9.0
BioSAK top_16S_hits -p 16S_vs_GTDB -q 16S_sequences.fa -r GTDB_ssu_all_r95.fna -t 6
BioSAK top_16S_hits -p 16S_vs_SILVA -q 16S_sequences.fa -r SILVA_138_SSURef_NR99_tax_silva.fasta -t 6 -top 5

# prepare GTDB SSU database file
wget https://data.gtdb.ecogenomic.org/releases/release95/95.0/genomic_files_all/ssu_all_r95.tar.gz
tar xvzf ssu_all_r95.tar.gz
makeblastdb -in GTDB_ssu_all_r95.fna -dbtype nucl -parse_seqids -logfile /dev/null

# prepare SILVA SSU database file
wget https://www.arb-silva.de/fileadmin/silva_databases/release_138/Exports/SILVA_138_SSURef_NR99_tax_silva.fasta.gz
gunzip SILVA_138_SSURef_NR99_tax_silva.fasta.gz
makeblastdb -in SILVA_138_SSURef_NR99_tax_silva.fasta -dbtype nucl -parse_seqids -logfile /dev/null

=============================================================================================================
'''


def keep_best_blast_hit(file_in, file_out):

    file_out_handle = open(file_out, 'w')
    best_hit_line = ''
    best_hit_query_id = ''
    best_hit_score = 0
    for blast_hit in open(file_in):
        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]
        bit_score = float(blast_hit_split[11])

        if best_hit_query_id == '':
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

        elif (query_id == best_hit_query_id) and (bit_score > best_hit_score):
            best_hit_score = bit_score
            best_hit_line = blast_hit

        elif query_id != best_hit_query_id:
            file_out_handle.write(best_hit_line)
            best_hit_query_id = query_id
            best_hit_line = blast_hit
            best_hit_score = bit_score

    file_out_handle.write(best_hit_line)
    file_out_handle.close()


def keep_top_n_blast_hit(file_in, file_out, top_hit_num):

    file_out_handle = open(file_out, 'w')

    current_query_id = ''
    current_query_wrote_hits = 0
    for blast_hit in open(file_in):

        blast_hit_split = blast_hit.strip().split('\t')
        query_id = blast_hit_split[0]

        if current_query_id == '':
            current_query_id = query_id
            file_out_handle.write(blast_hit)
            current_query_wrote_hits += 1

        elif current_query_id != '':
            if query_id == current_query_id:
                if current_query_wrote_hits < top_hit_num:
                    file_out_handle.write(blast_hit)
                    current_query_wrote_hits += 1
            else:
                current_query_id = query_id
                file_out_handle.write(blast_hit)
                current_query_wrote_hits = 1

    file_out_handle.close()


def top_16S_hits(args):

    output_prefix       = args['p']
    query_seq           = args['q']
    gtdb_ssu_file       = args['r']
    num_threads         = args['t']
    evalue_cutoff       = args['evalue']
    top_hit_num         = args['top']
    previous_blast_op   = args['b']

    # define output file name
    blast_op            = '%s_blastn.tab'           % output_prefix
    blast_op_best_hit   = '%s_blastn_best_hit.tab'  % output_prefix
    op_classified       = '%s_classifications.txt'  % output_prefix
    op_unclassified     = '%s_unclassified.txt'     % output_prefix

    # get_query_seq_list
    print(datetime.now().strftime(time_format) + 'Get the id of query sequences')
    query_seq_set = set()
    for each_query in open(query_seq):
        if each_query.startswith('>'):
            query_id = each_query.strip()[1:].split(' ')[0]
            query_seq_set.add(query_id)

    # get ref_seq_to_taxon_dict
    print(datetime.now().strftime(time_format) + 'Read in database file')
    ref_seq_to_taxon_dict = {}
    for each_line in open(gtdb_ssu_file):
        if each_line.startswith('>'):

            each_line_split = each_line.strip()[1:].split(' ')
            ref_seq_id = each_line_split[0]

            # get ref_seq_des
            ref_seq_des = ' '.join(each_line_split[1:])
            if ' [' in ref_seq_des:
                ref_seq_des = ref_seq_des.split(' [')[0]
            if ' ' in ref_seq_des:
                ref_seq_des = '_'.join(ref_seq_des.split(' '))

            ref_seq_to_taxon_dict[ref_seq_id] = ref_seq_des


    # run blast and keep best hits
    if previous_blast_op is None:
        print(datetime.now().strftime(time_format) + 'Running blast with %s cores, be patient!' % num_threads)
        blastn_cmd = 'blastn -query %s -db %s -out %s -outfmt 6 -evalue %s -num_threads %s' % (query_seq, gtdb_ssu_file, blast_op, evalue_cutoff, num_threads)
        os.system(blastn_cmd)
    else:
        blast_op = previous_blast_op

    print(datetime.now().strftime(time_format) + 'Keep the top %s hits for each query' % top_hit_num)
    if top_hit_num == 1:
        keep_best_blast_hit(blast_op, blast_op_best_hit)
    else:
        keep_top_n_blast_hit(blast_op, blast_op_best_hit, top_hit_num)

    # Parse blast results
    print(datetime.now().strftime(time_format) + 'Parse blast results')
    op_classified_handle = open(op_classified, 'w')
    op_classified_handle.write('Query\tReference\tIdentity\tAligned_length\tTaxonomy\n')
    classified_queries = set()
    for each_hit in open(blast_op_best_hit):
        each_hit_split  = each_hit.strip().split('\t')
        query_id        = each_hit_split[0]
        ref_id          = each_hit_split[1]
        identity        = float(each_hit_split[2])
        aln_len         = int(each_hit_split[3])
        ref_taxon       = ref_seq_to_taxon_dict.get(ref_id, 'NA')
        if ref_taxon != 'NA':
            op_classified_handle.write('%s\t%s\t%s\t%s\t%s\n' % (query_id, ref_id, identity, aln_len, ref_taxon))
            classified_queries.add(query_id)
    op_classified_handle.close()

    # write out id of unclassified queries
    unclassified_queries = query_seq_set - classified_queries
    if len(unclassified_queries) > 0:
        op_unclassified_handle = open(op_unclassified, 'w')
        op_unclassified_handle.write('\n'.join(unclassified_queries))
        op_unclassified_handle.close()

    # final report
    print(datetime.now().strftime(time_format) + 'Classifications exported to %s'        % op_classified)
    if len(unclassified_queries) > 0:
        print(datetime.now().strftime(time_format) + 'Unclassified sequences exported to %s'  % op_unclassified)
    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == '__main__':

    top_16S_hits_parser = argparse.ArgumentParser(usage=top_16S_hits_usage)
    top_16S_hits_parser.add_argument('-p',           required=True,                           help='output prefix')
    top_16S_hits_parser.add_argument('-q',           required=True,                           help='query sequence file')
    top_16S_hits_parser.add_argument('-r',           required=True,                           help='SILVA or GTDB SSU sequence file')
    top_16S_hits_parser.add_argument('-b',           required=False, default=None,            help='blast results between query and reference sequences from previous analysis(if you have)')
    top_16S_hits_parser.add_argument('-evalue',      required=False, default='1e-20',         help='evalue cutoff, default: 1e-20')
    top_16S_hits_parser.add_argument('-top',         required=False, type=int, default=1,     help='Number of top hits to report, default: 1')
    top_16S_hits_parser.add_argument('-t',           required=False, type=int, default=1,     help='number of threads')
    args = vars(top_16S_hits_parser.parse_args())
    top_16S_hits(args)



import os
import glob
import argparse
from Bio import SeqIO
from itertools import groupby


magabund_usage = '''
=============================== magabund example commands ===============================

# example command 
BioSAK magabund -s all_bins.sam -m all_bins -x fa -o output.txt

# Note
The input sam file is obtained by mapping quality-filtered reads to the assemblies 
derived from these reads. 

=========================================================================================
'''


def Cdb_2_bin_cluster_file(Cdb_file, bin_cluster_file):
    cluster_to_bin_dict = {}
    obtained_clusters = set()
    for each_bin in open(Cdb_file):
        if not each_bin.startswith('genome,secondary_cluster'):
            each_bin_split = each_bin.strip().split(',')
            bin_id = each_bin_split[0]
            secondary_cluster = each_bin_split[1]
            obtained_clusters.add(secondary_cluster)
            if secondary_cluster not in cluster_to_bin_dict:
                cluster_to_bin_dict[secondary_cluster] = [bin_id]
            else:
                cluster_to_bin_dict[secondary_cluster].append(bin_id)

    obtained_clusters_list = sorted([i for i in obtained_clusters])

    bin_cluster_file_handle = open(bin_cluster_file, 'w')
    for j in obtained_clusters_list:
        bin_cluster_file_handle.write('cluster_%s\t%s\n' % (j, '\t'.join(cluster_to_bin_dict[j])))
    bin_cluster_file_handle.close()


def cigar_to_read_len(cigar_string):

    # Given a CIGAR string, return the number of bases consumed from the query sequence
    read_consuming_ops = ("M", "I", "S", "=", "X")
    result = 0
    cig_iter = groupby(cigar_string, lambda chr: chr.isdigit())
    for _, length_digits in cig_iter:
        length = int(''.join(length_digits))
        op = next(next(cig_iter)[1])
        if op in read_consuming_ops:
            result += length
    return result


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if (each_element.isalpha() is True) or (each_element == '='):
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


def check_both_ends_clipping(cigar_splitted):

    both_ends_clipping = False
    if len(cigar_splitted) >= 3:
        if (cigar_splitted[0][-1] in ['S', 's']) and (cigar_splitted[-1][-1] in ['S', 's']):
            both_ends_clipping = True

    return both_ends_clipping


def get_cigar_stats(cigar_splitted):

    # aligned_len: M I X =
    # clipping_len: S
    # mismatch_len: X I D
    # mismatch_pct = mismatch_len / aligned_len
    # aligned_pct  = aligned_len  / (aligned_len + clipping_len)
    # clipping_pct = clipping_len / (aligned_len + clipping_len)

    aligned_len = 0
    clipping_len = 0
    mismatch_len = 0
    for each_part in cigar_splitted:
        each_part_len = int(each_part[:-1])
        each_part_cate = each_part[-1]

        # get aligned_len
        if each_part_cate in {'M', 'm', 'I', 'i', 'X', 'x', '='}:
            aligned_len += each_part_len

        # get clipping_len
        if each_part_cate in ['S', 's']:
            clipping_len += each_part_len

        # get mismatch_len
        if each_part_cate in {'I', 'i', 'X', 'x', 'D', 'd'}:
            mismatch_len += each_part_len

    aligned_pct  = float("{0:.2f}".format(aligned_len * 100 / (aligned_len + clipping_len)))
    clipping_pct = float("{0:.2f}".format(clipping_len * 100 / (aligned_len + clipping_len)))
    mismatch_pct = float("{0:.2f}".format(mismatch_len * 100 / (aligned_len)))

    return aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct


# def get_ref_to_total_read_len_from_sam(input_sam_file, ctg2gnm_dict, read2len_dict):
#
#     ref_len_dict = dict()
#     reads_in_binned = set()
#     read_in_sam = set()
#     gnm_to_aligned_read_dict = dict()
#     for each_read in open(input_sam_file):
#         each_read_split = each_read.strip().split('\t')
#         if each_read.startswith('@'):
#             ref_id = ''
#             ref_len = 0
#             for each_element in each_read_split:
#                 if each_element.startswith('SN:'):
#                     ref_id = each_element[3:]
#                 if each_element.startswith('LN:'):
#                     ref_len = int(each_element[3:])
#             ref_len_dict[ref_id] = ref_len
#         else:
#             read_id   = each_read_split[0]
#             ref_id    = each_read_split[2]
#             ref_pos   = int(each_read_split[3])
#             cigar     = each_read_split[5]
#             cigar_splitted = cigar_splitter(cigar)
#             read_in_sam.add(read_id)
#             if ref_id == '*':
#                 gnm_id = 'unmapped'
#             else:
#                 gnm_id = ctg2gnm_dict.get(ref_id, 'unbinned')
#                 if gnm_id != 'unbinned':
#                     reads_in_binned.add(read_id)
#
#             to_ignore = False
#             both_ends_clipping = check_both_ends_clipping(cigar_splitted)
#             if both_ends_clipping is True:
#                 left_clip_len = int(cigar_splitted[0][:-1])
#                 left_clip_pct = left_clip_len*100/read2len_dict[read_id]
#                 right_clip_len = int(cigar_splitted[-1][:-1])
#                 right_clip_pct = right_clip_len*100/read2len_dict[read_id]
#                 if (left_clip_pct >=5) and (right_clip_pct >=5) and (left_clip_len >= 50) and (right_clip_len >=50):
#                     to_ignore = True
#
#             # check if clp in the middle
#             if ref_id == '*':
#                 clip_in_middle = False
#             else:
#                 aln_len, aln_pct, clp_len, clp_pct, mismatch_pct = get_cigar_stats(cigar_splitted)
#                 ref_len = ref_len_dict[ref_id]
#                 clip_in_middle = False
#                 if ('S' in cigar) or ('s' in cigar):
#                     if (cigar_splitted[0][-1] in ['S', 's']) and (ref_pos > 200):
#                         clip_in_middle = True
#                     if (cigar_splitted[-1][-1] in ['S', 's']):
#                         if (ref_pos + aln_len - 1) - ref_len > 200:
#                             clip_in_middle = True
#
#             if (to_ignore is False) and (clip_in_middle is False):
#                 if gnm_id not in gnm_to_aligned_read_dict:
#                     gnm_to_aligned_read_dict[gnm_id] = {read_id}
#                 else:
#                     gnm_to_aligned_read_dict[gnm_id].add(read_id)
#
#     unbinned_updated = set()
#     for each_unbinned_read in gnm_to_aligned_read_dict.get('unbinned', set()):
#         if each_unbinned_read not in reads_in_binned:
#             unbinned_updated.add(each_unbinned_read)
#     gnm_to_aligned_read_dict['unbinned'] = unbinned_updated
#
#     ###############
#
#     gnm_to_aligned_read_len_dict = dict()
#     for each_gnm in gnm_to_aligned_read_dict:
#         gnm_to_aligned_read_len_dict[each_gnm] = 0
#         for each_read in gnm_to_aligned_read_dict[each_gnm]:
#             read_len = read2len_dict[each_read]
#             gnm_to_aligned_read_len_dict[each_gnm] += read_len
#
#     return gnm_to_aligned_read_len_dict, read_in_sam


def get_ref_to_total_read_len_from_sam(input_sam_file, ctg2gnm_dict, read2len_dict):

    read_in_sam = set()
    reads2gnm_dict = dict()
    for each_read in open(input_sam_file):
        if not each_read.startswith('@'):
            each_read_split = each_read.strip().split('\t')
            read_id = each_read_split[0]
            ref_id = each_read_split[2]
            ref_pos = int(each_read_split[3])
            cigar = each_read_split[5]
            read_in_sam.add(read_id)
            if ref_id == '*':
                gnm_id = 'unmapped'
            else:
                gnm_id = ctg2gnm_dict.get(ref_id, 'unbinned')

            if read_id not in reads2gnm_dict:
                reads2gnm_dict[read_id] = dict()
                reads2gnm_dict[read_id][gnm_id] = [cigar]
            else:
                if gnm_id not in reads2gnm_dict[read_id]:
                    reads2gnm_dict[read_id][gnm_id] = [cigar]
                else:
                    reads2gnm_dict[read_id][gnm_id].append(cigar)

    gnm_to_aligned_read_dict = dict()
    for each_read in reads2gnm_dict:
        mapped_gnms = reads2gnm_dict[each_read]

        if len(mapped_gnms) == 1:
            for mapped_gnm in mapped_gnms:
                if mapped_gnm not in gnm_to_aligned_read_dict:
                    gnm_to_aligned_read_dict[mapped_gnm] = {each_read}
                else:
                    gnm_to_aligned_read_dict[mapped_gnm].add(each_read)
        elif (len(mapped_gnms) == 2) and ('unbinned' in mapped_gnms):
            for mapped_gnm in mapped_gnms:
                if mapped_gnm != 'unbinned':
                    if mapped_gnm not in gnm_to_aligned_read_dict:
                        gnm_to_aligned_read_dict[mapped_gnm] = {each_read}
                    else:
                        gnm_to_aligned_read_dict[mapped_gnm].add(each_read)
        else:
            gnm_with_longest_aln_len = ''
            longest_aln_len = 0
            for mapped_gnm in mapped_gnms:
                if mapped_gnm != 'unbinned':
                    for each_cigar in mapped_gnms[mapped_gnm]:
                        cigar_splitted = cigar_splitter(each_cigar)
                        aln_len, aln_pct, clp_len, clp_pct, mismatch_pct = get_cigar_stats(cigar_splitted)
                        if aln_len > longest_aln_len:
                            longest_aln_len = aln_len
                            gnm_with_longest_aln_len = mapped_gnm

            if gnm_with_longest_aln_len not in gnm_to_aligned_read_dict:
                gnm_to_aligned_read_dict[gnm_with_longest_aln_len] = {each_read}
            else:
                gnm_to_aligned_read_dict[gnm_with_longest_aln_len].add(each_read)

    gnm_to_aligned_read_len_dict = dict()
    for each_gnm in gnm_to_aligned_read_dict:
        gnm_to_aligned_read_len_dict[each_gnm] = 0
        for each_read in gnm_to_aligned_read_dict[each_gnm]:
            read_len = read2len_dict[each_read]
            gnm_to_aligned_read_len_dict[each_gnm] += read_len

    return gnm_to_aligned_read_len_dict, read_in_sam


def magabund(args):

    ################################################# read in arguments ################################################

    sam_file      = args['s']
    bin_folder    = args['m']
    bin_ext       = args['x']
    output_file   = args['o']
    only_aligned  = args['a']
    # cluster_info  = args['g']
    # dRep_Cdb_file = args['Cdb']

    ############################################## define bin_cluster file #############################################

    # bin_cluster_file = ''
    # if (cluster_info is None) and (dRep_Cdb_file is None):
    #     bin_cluster_file = None
    #
    # elif (cluster_info is not None) and (dRep_Cdb_file is None):
    #     bin_cluster_file = cluster_info
    #
    # elif (cluster_info is None) and (dRep_Cdb_file is not None):
    #     Cdb_file_path, Cdb_file_basename, Cdb_file_extension = sep_path_basename_ext(dRep_Cdb_file)
    #     cluster_file_from_Cdb = '%s/%s_derived_cluster_file_%s%s' % (Cdb_file_path, Cdb_file_basename, datetime.now().strftime('%Y-%m-%d_%Hh-%Mm-%Ss_%f'), Cdb_file_extension)
    #     Cdb_2_bin_cluster_file(dRep_Cdb_file, cluster_file_from_Cdb)
    #     bin_cluster_file = cluster_file_from_Cdb
    # else:
    #     print(datetime.now().strftime(time_format) + 'cluster_info and dRep_Cdb are not compatible, please specify one only, program exited!')
    #     exit()

    ################################################ get ctg to bin dict ###############################################

    bin_file_re = '%s/*%s' % (bin_folder, bin_ext)
    bin_file_list = [os.path.basename(file_name) for file_name in glob.glob(bin_file_re)]

    if len(bin_file_list) == 0:
        print('No genome found, program exited!')
        exit()

    bin_size_dict = dict()
    ctg_2_bin_dict = dict()
    counted_ctg_set = set()
    ctgs_found_in_multiple_mags = set()
    for each_bin in bin_file_list:
        pwd_each_bin = '%s/%s' % (bin_folder, each_bin)
        bin_size_dict[each_bin] = 0
        for seq in SeqIO.parse(pwd_each_bin, 'fasta'):
            ctg_2_bin_dict[seq.id] = each_bin
            bin_size_dict[each_bin] += len(seq.seq)
            if seq.id not in counted_ctg_set:
                counted_ctg_set.add(seq.id)
            else:
                ctgs_found_in_multiple_mags.add(seq.id)

    if len(ctgs_found_in_multiple_mags) > 0:
        print('The following %s sequences found in multiple genomes, program exited!' % len(ctgs_found_in_multiple_mags))
        print(','.join(ctgs_found_in_multiple_mags))
        exit()

    ############################################ get ctg to group list dict ############################################

    # print(datetime.now().strftime(time_format) + 'Get bin (cluster) to contig correlations')
    #
    # if bin_cluster_file is None:
    #     group_2_ctg_dict = bin_2_ctg_dict
    # else:
    #     # get group to bin dict
    #     group_2_bin_dict = {}
    #     for group in open(bin_cluster_file):
    #         group_split = group.strip().split('\t')
    #         group_2_bin_dict[group_split[0]] = group_split[1:]
    #
    #     group_2_ctg_dict = {}
    #     for bin_group in group_2_bin_dict:
    #         group_member_list = group_2_bin_dict[bin_group]
    #         group_2_ctg_dict[bin_group] = set()
    #         for genome_bin in group_member_list:
    #             genome_bin_ctg_list = bin_2_ctg_dict[genome_bin]
    #             for ctg in genome_bin_ctg_list:
    #                 group_2_ctg_dict[bin_group].add(ctg)

    ########################################### get_ref_to_read_num_from_sam ###########################################

    reads_file = 'assembly_graph_NoEu.gfa_reads_min0bp.fastq'

    read_len_dict = dict()
    for each_read in SeqIO.parse(reads_file, 'fastq'):
        read_len_dict[each_read.id] = len(each_read.seq)

    print('Get the total length of reads mapped to each reference sequence in sam file')

    gnm_to_aligned_read_total_len_dict, reads_in_sam = get_ref_to_total_read_len_from_sam(sam_file, ctg_2_bin_dict, read_len_dict)

    reads_not_in_sam_total_len = 0
    for each_read in read_len_dict:
        read_len = read_len_dict[each_read]
        if each_read not in reads_in_sam:
            reads_not_in_sam_total_len += read_len
    print('reads_not_in_sam_total_len\t%s' % reads_not_in_sam_total_len)

    if 'unmapped' not in gnm_to_aligned_read_total_len_dict:
        gnm_to_aligned_read_total_len_dict['unmapped'] = reads_not_in_sam_total_len
    else:
        gnm_to_aligned_read_total_len_dict['unmapped'] += reads_not_in_sam_total_len


    total_read_len    = sum(gnm_to_aligned_read_total_len_dict.values())
    total_gnm_size_bp = sum(bin_size_dict.values())
    gnm_list          = sorted(list(gnm_to_aligned_read_total_len_dict.keys()))

    # get total_mag_depth
    total_mag_depth = 0
    for each_gnm in gnm_list:
        if each_gnm not in ['unbinned', 'unmapped']:
            gnm_size_bp     = bin_size_dict[each_gnm]
            gnm_reads_len   = gnm_to_aligned_read_total_len_dict[each_gnm]
            gnm_reads_depth = float("{0:.2f}".format(gnm_reads_len/gnm_size_bp))
            total_mag_depth += gnm_reads_depth
    total_mag_depth = float("{0:.2f}".format(total_mag_depth))

    # write out table
    gnm_reads_len_pct_total = 0
    gnm_reads_depth_norm_total = 0
    output_file_handle = open(output_file, 'w')
    output_file_handle.write('Genome\tGenome_size(bp)\tMapped_reads_length(bp)\tMapped_reads_percent\tDepth(x)\tDepth/%s\n' % total_mag_depth)
    for each_gnm in gnm_list:
        if each_gnm not in ['unbinned', 'unmapped']:
            gnm_size_bp                 = bin_size_dict[each_gnm]
            gnm_reads_len               = gnm_to_aligned_read_total_len_dict[each_gnm]
            gnm_reads_len_pct           = float("{0:.2f}".format(gnm_reads_len*100/total_read_len))
            gnm_reads_depth             = float("{0:.2f}".format(gnm_reads_len/gnm_size_bp))
            gnm_reads_depth_norm        = float("{0:.2f}".format(gnm_reads_depth*100/total_mag_depth))
            gnm_reads_len_pct_total     += gnm_reads_len_pct
            gnm_reads_depth_norm_total  += gnm_reads_depth_norm
            output_file_handle.write('%s\t%s\t%s\t%s\t%s\t%s\n' % (each_gnm, gnm_size_bp, gnm_reads_len, gnm_reads_len_pct, gnm_reads_depth, gnm_reads_depth_norm))

    if 'unbinned' in gnm_to_aligned_read_total_len_dict:
        gnm_reads_len           = gnm_to_aligned_read_total_len_dict['unbinned']
        gnm_reads_len_pct       = float("{0:.2f}".format(gnm_reads_len*100/total_read_len))
        gnm_reads_len_pct_total += gnm_reads_len_pct
        output_file_handle.write('%s\tNA\t%s\t%s\tNA\tNA\n' % ('unbinned', gnm_reads_len, gnm_reads_len_pct))

    if 'unmapped' in gnm_to_aligned_read_total_len_dict:
        gnm_reads_len           = gnm_to_aligned_read_total_len_dict['unmapped']
        gnm_reads_len_pct       = float("{0:.2f}".format(gnm_reads_len*100/total_read_len))
        gnm_reads_len_pct_total += gnm_reads_len_pct
        output_file_handle.write('%s\tNA\t%s\t%s\tNA\tNA\n' % ('unmapped', gnm_reads_len, gnm_reads_len_pct))

    output_file_handle.write('Total\t%s\t%s\t%s\t%s\t%s\n' % (total_gnm_size_bp, total_read_len, gnm_reads_len_pct_total, float("{0:.2f}".format(total_mag_depth)), gnm_reads_depth_norm_total))
    output_file_handle.close()

    # final report
    print('Done!')


if __name__ == "__main__":

    magabund_parser = argparse.ArgumentParser()
    magabund_parser.add_argument('-s',     required=True,                        help='input sam file')
    magabund_parser.add_argument('-m',     required=True,                        help='MAG folder')
    magabund_parser.add_argument('-x',     required=True, default='fasta',       help='MAG file extension, default: fasta')
    magabund_parser.add_argument('-o',     required=True,                        help='output table')
    magabund_parser.add_argument('-a',     required=False, action='store_true',  help='based on aligned length, rather than read length')
    #magabund_parser.add_argument('-g',       required=False, default=None,         help='bin grouping info')
    #magabund_parser.add_argument('-Cdb',     required=False, default=None,         help='cluster info from dRep (Cdb.csv)')
    args = vars(magabund_parser.parse_args())
    magabund(args)

'''
# get abundance of customized bin clusters
BioSAK get_bin_abundance -sam all_bins.sam -bin all_bins -x fa -o output.txt -g bin_clusters.txt

# get abundance of dRep produced bin clusters
BioSAK get_bin_abundance -sam all_bins.sam -bin all_bins -x fa -o output.txt -Cdb Cdb.csv

cd /Users/songweizhi/Desktop/get_MAG_abundance_wd
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/magabund.py -sam three_drep_mags.sam -bin mags -x fa -o test.txt

# How it works
The get_bin_abundance module first gets the number of reads mapped to each reference sequence in the provided sam file. 
Then calculate the number/percentage of reads mapped to the sequences in each bin (cluster).

# Format of customized grouping file (tab-separated, with the first column as group id, followed by a list of bins from it)
cluster_1	Ecklonia_bin_1.fa	Delisea_bin_1.fa
cluster_2	Ecklonia_bin_2.fa	Delisea_bin_5.fa	Amphiroa_bin_1.fa
cluster_3	Amphiroa_bin_3.fa

'''



import argparse
from Bio import SeqIO
from BioSAK.global_functions import sep_path_basename_ext


SILVA_for_BLCA_usage = '''
================= SILVA_for_BLCA example commands =================

BioSAK SILVA_for_BLCA -i SILVA_138.2_SSURef_NR99_tax_silva.fasta

# output_file:
SILVA_138.2_SSURef_NR99_tax_silva.blca.fa
SILVA_138.2_SSURef_NR99_tax_silva.blca.tax

===================================================================
'''


def SILVA_for_BLCA(args):

    SILVA_db_file = args['i']

    db_path, db_base, db_ext = sep_path_basename_ext(SILVA_db_file)
    file_out_sequence = '%s/%s.blca.fa'     % (db_path, db_base)
    file_out_taxonomy = '%s/%s.blca.tax'    % (db_path, db_base)

    rank_list = ['species', 'genus', 'family', 'order', 'class', 'phylum', 'superkingdom']

    file_out_sequence_handle = open(file_out_sequence, 'w')
    file_out_taxonomy_handle = open(file_out_taxonomy, 'w')
    for SILVA_seq in SeqIO.parse(SILVA_db_file, 'fasta'):

        SILVA_seq_taxon = ' '.join(SILVA_seq.description.split(' ')[1:])

        if not SILVA_seq_taxon.startswith('Eukaryota'):

            SILVA_seq_taxon_split = SILVA_seq_taxon.split(';')

            if len(SILVA_seq_taxon_split) < 7:

                if len(SILVA_seq_taxon_split) == 6:
                    SILVA_seq_taxon_split.append('unidentified')
                if len(SILVA_seq_taxon_split) == 5:
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                if len(SILVA_seq_taxon_split) == 4:
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                if len(SILVA_seq_taxon_split) == 3:
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                if len(SILVA_seq_taxon_split) == 2:
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')
                    SILVA_seq_taxon_split.append('unidentified')

            if len(SILVA_seq_taxon_split) > 7:
                SILVA_seq_taxon_split = [SILVA_seq_taxon_split[0], SILVA_seq_taxon_split[1], SILVA_seq_taxon_split[2], SILVA_seq_taxon_split[3], SILVA_seq_taxon_split[4], SILVA_seq_taxon_split[5], ' '.join(SILVA_seq_taxon_split[6:])]

            SILVA_seq_taxon_split_reverse = SILVA_seq_taxon_split[::-1]

            SILVA_seq_taxon_str = ''
            n = 0
            for taxon_rank in rank_list:
                SILVA_seq_taxon_str += '%s:%s;' % (taxon_rank, SILVA_seq_taxon_split_reverse[n])
                n += 1

            # write out to taxonomy file
            file_out_taxonomy_handle.write('%s\t%s\n' % (SILVA_seq.id, SILVA_seq_taxon_str))

            # write out to sequence file
            file_out_sequence_handle.write('>%s\n' % SILVA_seq.description)
            file_out_sequence_handle.write('%s\n' % SILVA_seq.seq)

    file_out_sequence_handle.close()
    file_out_taxonomy_handle.close()

    print('Done!')
    print('You may need to run:')
    print('makeblastdb -dbtype nucl -parse_seqids -in %s' % file_out_sequence)


if __name__ == "__main__":

    parser = argparse.ArgumentParser(usage=SILVA_for_BLCA_usage)
    parser.add_argument('-i', required=True, help='SILVA SSU file, e.g. SILVA_138.2_SSURef_NR99_tax_silva.fasta')
    args = vars(parser.parse_args())
    SILVA_for_BLCA(args)



import argparse
import os

get_single_page_web_usage = '''
================================ get_single_page_web example commands ================================

BioSAK get_single_page_web -d doc/images -m doc/metadata.txt -w doc/template.html -o single_page_web.html
BioSAK get_single_page_web -d doc/images -m doc/metadata.txt -w doc/template_with_barchart.html -o single_page_web_with_barchart.html

# Note
There should be no parentheses in metadata

======================================================================================================
'''


def get_single_page_web(args):

    sample_metadata_txt    = args['m']
    web_page_file_template = args['w']
    photo_dir              = args['d']
    web_page_file_out      = args['o']

    # read in sample metadata
    photo_section_str = ''
    col_index = dict()
    line_num_index = 0
    for each_line in open(sample_metadata_txt):
        line_num_index += 1
        line_split = each_line.strip().split('\t')
        if line_num_index == 1:
            col_index = {key: i for i, key in enumerate(line_split)}
        else:
            sample_accession    = line_split[col_index['Accession']]
            sample_image        = line_split[col_index['Image']]
            sample_taxon        = line_split[col_index['Taxon']]
            sample_location     = ''
            sample_location_eng = line_split[col_index['Location_Eng']]
            sample_location_chi = line_split[col_index['Location_Chi']]
            sample_latitude     = line_split[col_index['Latitude']]
            sample_longitude    = line_split[col_index['Longitude']]
            sample_source       = line_split[col_index['Source']]
            sample_desc         = line_split[col_index['Description']]
            sample_date         = line_split[col_index['Date']]
            sample_link         = line_split[col_index['Link']]
            loci_with_coord     = '%s (%s/%s)' % (sample_location, sample_latitude, sample_longitude)
            sample_coord        = '%s/%s' % (sample_latitude, sample_longitude)

            pwd_photo = sample_image
            if photo_dir is not None:
                pwd_photo = '%s/%s' % (photo_dir, sample_image)

            photo_attribute_str = '      {accession: "%s", photo: "%s", taxon: "%s", loci: "%s", loci_eng: "%s", loci_chi: "%s", lat: %s, lng: %s, desc: "%s", coord: "%s", source: "%s", date: "%s", link: "%s"},' % (sample_accession, pwd_photo, sample_taxon, sample_location, sample_location_eng, sample_location_chi, sample_latitude, sample_longitude, sample_desc, sample_coord, sample_source, sample_date, sample_link)
            photo_section_str += photo_attribute_str
            photo_section_str += '\n'

    # write out
    web_page_file_out_handle = open(web_page_file_out, 'w')
    for each_line in open(web_page_file_template):
        if '// add photo attributes here' not in each_line:
            web_page_file_out_handle.write(each_line)
        else:
            web_page_file_out_handle.write(photo_section_str)
    web_page_file_out_handle.close()


if __name__ == '__main__':
    get_single_page_web_parser = argparse.ArgumentParser(usage=get_single_page_web_usage)
    get_single_page_web_parser.add_argument('-m', required=True,                help='sample metadata')
    get_single_page_web_parser.add_argument('-w', required=True,                help='webpage template')
    get_single_page_web_parser.add_argument('-d', required=False, default=None, help='photo directory')
    get_single_page_web_parser.add_argument('-o', required=True,                help='output webpage')
    args = vars(get_single_page_web_parser.parse_args())
    get_single_page_web(args)



import os
import argparse


stats_blast_hit_usage = '''
================================= stats_blast_hit example commands =================================

BioSAK stats_blast_hit -i otu_vs_nt.txt -o stats.txt -node nodes.dmp -db RefSeq_taxonomy.txt -n 10

python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/stats_blast_hit.py -i /Users/songweizhi/Desktop/SMP/02_Usearch_BLCA_GTDB/Unclassified_3696_vs_nt.txt -o /Users/songweizhi/Desktop/SMP/02_Usearch_BLCA_GTDB/Unclassified_3696_vs_nt_stats.txt -n 20 -node /Users/songweizhi/DB/taxdump_20250321/nodes.dmp -db /Users/songweizhi/DB/NCBI/RefSeq_taxonomy.txt

====================================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def group_tax_by_domain(nodes_dmp):
    child_to_parent_dict = dict()
    for each_node in open(nodes_dmp):
        each_node_split = each_node.strip().split('|')
        tax_id = int(each_node_split[0].strip())
        parent_tax_id = int(each_node_split[1].strip())
        child_to_parent_dict[tax_id] = parent_tax_id

    tax_id_set_bac = set()
    tax_id_set_ar = set()
    tax_id_set_eu = set()
    tax_id_set_virus = set()
    for tax_id in child_to_parent_dict.keys():
        # get tax_lineage_list
        tax_lineage_list = [tax_id]
        while tax_id != 1:
            tax_id = child_to_parent_dict[tax_id]
            tax_lineage_list.append(tax_id)
        # Bacteria
        if 2 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_bac.add(each_id)
        # Archaea
        elif 2157 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_ar.add(each_id)
        # Eukaryota
        elif 2759 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_eu.add(each_id)
        # Virus
        elif 10239 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_virus.add(each_id)
    return tax_id_set_bac, tax_id_set_ar, tax_id_set_eu, tax_id_set_virus


def stats_blast_hit(args):

    blast_op_txt    = args['i']
    hit_stats_txt   = args['o']
    nodes_dmp       = args['node']
    refseq_tax_txt  = args['db']
    hit_num_to_keep = args['n']
    pct_cutoff      = args['c']

    f_name, f_path, f_base, f_ext = sep_path_basename_ext(hit_stats_txt)
    cmd_txt = '%s/%s_RefSeq_classification.sh' % (f_path, f_base)
    tax_set_bac, tax_set_ar, tax_set_eu, tax_set_virus = group_tax_by_domain(nodes_dmp)

    refseq_tax_dict = dict()
    for each_ref in open(refseq_tax_txt):
        each_ref_split = each_ref.strip().split('\t')
        if len(each_ref_split) == 2:
            refseq_tax_dict[each_ref_split[0]] = each_ref_split[1]

    query_to_hit_dict = dict()
    for each_line in open(blast_op_txt):
        each_line_split = each_line.strip().split('\t')
        query_id = each_line_split[0]
        subject_id = each_line_split[1]
        if query_id not in query_to_hit_dict:
            query_to_hit_dict[query_id] = set()
        if len(query_to_hit_dict[query_id]) < hit_num_to_keep:
            query_to_hit_dict[query_id].add(subject_id)

    hit_stats_txt_handle = open(hit_stats_txt, 'w')
    hit_stats_txt_handle.write('Query\tArchaea\tBacteria\tEukaryote\tVirus\tRest\tUnknown\tClassification\n')
    unclassified_refseq_set = set()
    for query in query_to_hit_dict:
        hit_tax_list = []
        for each_hit in query_to_hit_dict[query]:
            hit_tax = refseq_tax_dict.get(each_hit, 'na')
            if hit_tax == 'na':
                hit_tax_list.append(hit_tax)
                unclassified_refseq_set.add(each_hit)
            else:
                hit_tax_list.append(int(hit_tax))

        hit_num_bac = 0
        hit_num_ar = 0
        hit_num_eu = 0
        hit_num_virus = 0
        hit_num_na = 0
        hit_num_rest = 0
        for hit_tax in hit_tax_list:
            if hit_tax in tax_set_bac:
                hit_num_bac += 1
            elif hit_tax in tax_set_ar:
                hit_num_ar += 1
            elif hit_tax in tax_set_eu:
                hit_num_eu += 1
            elif hit_tax in tax_set_virus:
                hit_num_virus += 1
            elif hit_tax == 'na':
                hit_num_na += 1
            else:
                hit_num_rest += 1

        total_hit_num = len(hit_tax_list)
        try:
            hit_pct_bac = int(hit_num_bac*100/total_hit_num)
        except:
            hit_pct_bac = float("{0:.2f}".format(hit_num_bac*100/total_hit_num))
        try:
            hit_pct_ar = int(hit_num_ar*100/total_hit_num)
        except:
            hit_pct_ar = float("{0:.2f}".format(hit_num_ar*100/total_hit_num))
        try:
            hit_pct_eu = int(hit_num_eu*100/total_hit_num)
        except:
            hit_pct_eu = float("{0:.2f}".format(hit_num_eu*100/total_hit_num))
        try:
            hit_pct_virus = int(hit_num_virus*100/total_hit_num)
        except:
            hit_pct_virus = float("{0:.2f}".format(hit_num_virus*100/total_hit_num))
        try:
            hit_pct_na = int(hit_num_na*100/total_hit_num)
        except:
            hit_pct_na = float("{0:.2f}".format(hit_num_na*100/total_hit_num))
        try:
            hit_pct_rest = int(hit_num_rest*100/total_hit_num)
        except:
            hit_pct_rest = float("{0:.2f}".format(hit_num_rest*100/total_hit_num))

        classification = 'Unclassified'
        if hit_pct_ar >= pct_cutoff:
            classification = 'd__Archaea;p__;c__;o__;f__;g__;s__'
        elif hit_pct_bac >= pct_cutoff:
            classification = 'd__Bacteria;p__;c__;o__;f__;g__;s__'
        elif hit_pct_eu >= pct_cutoff:
            classification = 'Eukaryote'
        elif hit_pct_virus >= pct_cutoff:
            classification = 'Virus'
        elif hit_pct_rest >= pct_cutoff:
            classification = 'Rest'
        elif hit_pct_na >= pct_cutoff:
            classification = 'Unknown'

        hit_stats_txt_handle.write('%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n' % (query, hit_pct_ar, hit_pct_bac, hit_pct_eu, hit_pct_virus, hit_pct_rest, hit_pct_na, classification))
        #hit_stats_txt_handle.write('%s\t%s\n' % (query, classification))
    hit_stats_txt_handle.close()
    print('Done!')


if __name__ == '__main__':

    stats_blast_hit_parser = argparse.ArgumentParser(usage=stats_blast_hit_usage)
    stats_blast_hit_parser.add_argument('-i',    required=True,                         help='blast result, outfmt need to be 6')
    stats_blast_hit_parser.add_argument('-o',    required=True,                         help='output stats')
    stats_blast_hit_parser.add_argument('-node', required=True,                         help='nodes.dmp ')
    stats_blast_hit_parser.add_argument('-db',   required=True,                         help='RefSeq taxonomy file')
    stats_blast_hit_parser.add_argument('-c',    required=False, type=int, default=90,  help='cutoff, default is 90')
    stats_blast_hit_parser.add_argument('-n',    required=False, type=int, default=20,  help='number of top hits to consider, default is 20')
    args = vars(stats_blast_hit_parser.parse_args())
    stats_blast_hit(args)



import os
import glob
from Bio import SeqIO


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def rename_and_cat_gnm(gnm_dir, gnm_ext, combined_renamed_fna, rename_txt):

    gnm_file_re = '%s/*.%s' % (gnm_dir, gnm_ext)
    gnm_file_list = glob.glob(gnm_file_re)

    gnm_rename_txt_handle = open(rename_txt, 'w')
    combined_renamed_fa_handle = open(combined_renamed_fna, 'w')
    for each_gnm in sorted(gnm_file_list):
        _, _, gnm_base, _ = sep_path_basename_ext(each_gnm)
        gnm_id_no_underscore = gnm_base.replace('_', '')
        gnm_rename_txt_handle.write('%s\t%s\n' % (gnm_base, gnm_id_no_underscore))

        seq_index = 1
        for each_seq in SeqIO.parse(each_gnm, 'fasta'):
            combined_renamed_fa_handle.write('>%s_%s\n%s\n' % (gnm_id_no_underscore, seq_index, str(each_seq.seq)))
            seq_index += 1

    gnm_rename_txt_handle.close()
    combined_renamed_fa_handle.close()


# file in
gnm_dir = '/Users/songweizhi/Desktop/3_combined_genomes_50_5_614_dRep99_406'
gnm_ext = 'fna'

# file out
combined_renamed_fa = '/Users/songweizhi/Desktop/Sponge_r220/11_host_specificity_by_abundance/dRep99_406_renamed_combined.fna'
gnm_rename_txt      = '/Users/songweizhi/Desktop/Sponge_r220/11_host_specificity_by_abundance/dRep99_406_renamed_combined.txt'

rename_and_cat_gnm(gnm_dir, gnm_ext, combined_renamed_fa, gnm_rename_txt)



import os
import argparse


sra2biosample_usage = '''
================= sra2biosample example commands =================

BioSAK sra2biosample -i SRR18249231 -o SRR18249231_biosample.txt
BioSAK sra2biosample -i sra.txt -o biosample.txt

==================================================================
'''


def sep_path_basename_ext(file_in):
    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]
    return f_name, f_path, f_base, f_ext


def sra_to_biosample_bioproject(sra_id_set, tmp_dir):

    # create tmp_dir
    if os.path.isdir(tmp_dir) is False:
        os.mkdir(tmp_dir)

    sra2biosample_dict = dict()
    sra2bioproject_dict = dict()
    for each_sra in sra_id_set:
        op_txt_biosample = '%s/%s_Biosample.txt' % (tmp_dir, each_sra)
        sra_to_biosample_cmd = 'esearch -db sra -query %s | efetch -format docsum | xtract -pattern DocumentSummary -element Biosample > %s' % (each_sra, op_txt_biosample)
        # print(sra_to_biosample_cmd)
        os.system(sra_to_biosample_cmd)
        with open(op_txt_biosample) as f:
            biosample_id = f.readline().strip('\n')
        sra2biosample_dict[each_sra] = biosample_id

        op_txt_bioproject = '%s/%s_Bioproject.txt' % (tmp_dir, each_sra)
        sra_to_bioproject_cmd = 'esearch -db sra -query %s | efetch -format docsum | xtract -pattern DocumentSummary -element Bioproject > %s' % (each_sra, op_txt_bioproject)
        # print(sra_to_bioproject_cmd)
        os.system(sra_to_bioproject_cmd)
        with open(op_txt_bioproject) as f:
            bioproject_id = f.readline().strip('\n')
        sra2bioproject_dict[each_sra] = bioproject_id

    return sra2biosample_dict, sra2bioproject_dict


def xml_to_txt(xml_file, txt_file):

    txt_file_handle = open(txt_file, 'w')
    for each_line in open(xml_file):
        each_line = each_line.strip()
        if (each_line.count('<') == 1) and (each_line.count('>') == 1):
            pass
        else:
            each_line = each_line.split('</')[0]
            title_str = each_line.split('>')[0][1:]
            value_str = each_line.split('>')[-1]
            if 'display_name' in title_str:
                title_str = title_str.split('display_name')[1].replace('=','').replace('"','')
            elif title_str.count('=') == 1:
                title_str = title_str.split('=')[1].replace('"','')
            elif title_str.count('=') == 0:
                title_str = title_str
            elif 'is_primary' in title_str:
                title_str = title_str.split('is_primary')[0].split('=')[1].replace('"','')
            txt_file_handle.write('%s\t%s\n' % (title_str, value_str))
    txt_file_handle.close()


def sra2biosample(args):

    file_in = args['i']
    op_txt  = args['o']

    accession_set = set()
    if os.path.isfile(file_in) is True:
        for each_id in open(file_in):
            accession_set.add(each_id.strip())
    else:
        accession_set.add(file_in)


    op_f_name, op_f_path, op_f_base, op_f_ext = sep_path_basename_ext(op_txt)

    tmp_dir = '%s/%s_sra2biosample_tmp_dir' % (op_f_path, op_f_base)

    # get sra_to_biosample_dict and sra_to_bioproject_dict
    sra_to_biosample_dict, sra_to_bioproject_dict = sra_to_biosample_bioproject(accession_set, tmp_dir)

    # write out
    op_txt_handle = open(op_txt, 'w')
    op_txt_handle.write('SRA\tBiosample\tBioproject\n')
    for each_sra_id in sorted(list(accession_set)):
        op_txt_handle.write('%s\t%s\t%s\n' % (each_sra_id, sra_to_biosample_dict.get(each_sra_id, 'na'), sra_to_bioproject_dict.get(each_sra_id, 'na')))
    op_txt_handle.close()


if __name__ == '__main__':

    arg_parser = argparse.ArgumentParser(usage=sra2biosample_usage)
    arg_parser.add_argument('-i',   required=True,                        help='input SRA id')
    arg_parser.add_argument('-o',   required=True,                        help='output file')
    args = vars(arg_parser.parse_args())
    sra2biosample(args)



import os
import glob
import argparse
from Bio import SeqIO
from datetime import datetime
from itertools import groupby
from BioSAK.global_functions import time_format
from BioSAK.global_functions import sep_path_basename_ext


get_bin_abundance_usage = '''
================================ get_bin_abundance example commands ================================

# get abundance of individual bins 
BioSAK get_bin_abundance -l -sam all_bins.sam -bin all_bins -x fa -o abundance.txt

# get abundance of customized bin clusters
BioSAK get_bin_abundance -l -sam all_bins.sam -bin all_bins -x fa -o abundance.txt -g bin_clusters.txt

# get abundance of dRep produced bin clusters
BioSAK get_bin_abundance -l -sam all_bins.sam -bin all_bins -x fa -o abundance.txt -Cdb Cdb.csv

# How it works
The get_bin_abundance module first gets the number of reads mapped to each reference sequence in the provided sam file. 
Then calculate the number/percentage of reads mapped to the sequences in each bin (cluster).

# Format of customized grouping file (tab-separated, with the first column as group id, followed by a list of bins from it)
cluster_1	Ecklonia_bin_1.fa	Delisea_bin_1.fa
cluster_2	Ecklonia_bin_2.fa	Delisea_bin_5.fa	Amphiroa_bin_1.fa
cluster_3	Amphiroa_bin_3.fa

# Note (not accurate!!!)
The input sam file is obtained by mapping sequencing reads from a sample to the combined file of all bins in ‘-bin’ folder. 
Please make sure contig ids are UNIQUE across all bins in ‘-bin’ folder.

====================================================================================================
'''


def Cdb_2_bin_cluster_file(Cdb_file, bin_cluster_file):
    cluster_to_bin_dict = {}
    obtained_clusters = set()
    for each_bin in open(Cdb_file):
        if not each_bin.startswith('genome,secondary_cluster'):
            each_bin_split = each_bin.strip().split(',')
            bin_id = each_bin_split[0]
            secondary_cluster = each_bin_split[1]

            obtained_clusters.add(secondary_cluster)

            if secondary_cluster not in cluster_to_bin_dict:
                cluster_to_bin_dict[secondary_cluster] = [bin_id]
            else:
                cluster_to_bin_dict[secondary_cluster].append(bin_id)

    obtained_clusters_list = sorted([i for i in obtained_clusters])

    bin_cluster_file_handle = open(bin_cluster_file, 'w')
    for j in obtained_clusters_list:
        bin_cluster_file_handle.write('cluster_%s\t%s\n' % (j, '\t'.join(cluster_to_bin_dict[j])))
    bin_cluster_file_handle.close()


def get_ref_to_read_num_from_sam(input_sam_file, output_stats_file):

    output_stats_file_path, output_stats_file_basename, output_stats_file_extension = sep_path_basename_ext(output_stats_file)
    output_stats_tmp = '%s/%s_tmp%s' % (output_stats_file_path, output_stats_file_basename, output_stats_file_extension)

    # Store reads num in dict
    ref2read_num_dict = {}
    for each_read in open(input_sam_file):
        if not each_read.startswith('@'):
            ref_id = each_read.strip().split('\t')[2]

            if ref_id not in ref2read_num_dict:
                ref2read_num_dict[ref_id] = 1
            else:
                ref2read_num_dict[ref_id] += 1

    # Write reads num to file
    stat_file_unsorted_handle = open(output_stats_tmp, 'w')
    for each_ref in ref2read_num_dict:
        stat_file_unsorted_handle.write('%s\t%s\n' % (each_ref, ref2read_num_dict[each_ref]))
    stat_file_unsorted_handle.close()

    # sort output file
    os.system('cat %s | sort > %s' % (output_stats_tmp, output_stats_file))

    # remove tmp file
    os.system('rm %s' % output_stats_tmp)


def cigar_to_read_len(cigar_string):
    # Given a CIGAR string, return the number of bases consumed from the query sequence
    read_consuming_ops = ("M", "I", "S", "=", "X")
    result = 0
    cig_iter = groupby(cigar_string, lambda chr: chr.isdigit())
    for _, length_digits in cig_iter:
        length = int(''.join(length_digits))
        op = next(next(cig_iter)[1])
        if op in read_consuming_ops:
            result += length
    return result


def get_ref_to_total_read_len_from_sam(input_sam_file, output_stats_file):

    output_stats_file_path, output_stats_file_basename, output_stats_file_extension = sep_path_basename_ext(output_stats_file)
    output_stats_tmp = '%s/%s_tmp%s' % (output_stats_file_path, output_stats_file_basename, output_stats_file_extension)

    # Store reads num in dict
    ref2read_num_dict = {}
    for each_read in open(input_sam_file):
        if not each_read.startswith('@'):
            each_read_split = each_read.strip().split('\t')
            ref_id = each_read_split[2]
            cigar_str = each_read_split[5]
            read_len = cigar_to_read_len(cigar_str)
            if ref_id not in ref2read_num_dict:
                ref2read_num_dict[ref_id] = read_len
            else:
                ref2read_num_dict[ref_id] += read_len

    # Write reads num to file
    stat_file_unsorted_handle = open(output_stats_tmp, 'w')
    for each_ref in ref2read_num_dict:
        stat_file_unsorted_handle.write('%s\t%s\n' % (each_ref, ref2read_num_dict[each_ref]))
    stat_file_unsorted_handle.close()

    # sort output file
    os.system('cat %s | sort > %s' % (output_stats_tmp, output_stats_file))

    # remove tmp file
    os.system('rm %s' % output_stats_tmp)


def get_bin_abundance(args):

    ################################################# read in arguments ################################################

    sam_file      = args['sam']
    bin_folder    = args['bin']
    bin_ext       = args['x']
    output_file   = args['o']
    cluster_info  = args['g']
    dRep_Cdb_file = args['Cdb']
    by_read_len   = args['l']

    ############################################## define bin_cluster file #############################################

    bin_cluster_file = ''
    if (cluster_info is None) and (dRep_Cdb_file is None):
        bin_cluster_file = None

    elif (cluster_info is not None) and (dRep_Cdb_file is None):
        bin_cluster_file = cluster_info

    elif (cluster_info is None) and (dRep_Cdb_file is not None):
        Cdb_file_path, Cdb_file_basename, Cdb_file_extension = sep_path_basename_ext(dRep_Cdb_file)
        cluster_file_from_Cdb = '%s/%s_derived_cluster_file_%s%s' % (Cdb_file_path, Cdb_file_basename, datetime.now().strftime('%Y-%m-%d_%Hh-%Mm-%Ss_%f'), Cdb_file_extension)
        Cdb_2_bin_cluster_file(dRep_Cdb_file, cluster_file_from_Cdb)
        bin_cluster_file = cluster_file_from_Cdb
    else:
        print(datetime.now().strftime(time_format) + 'cluster_info and dRep_Cdb are not compatible, please specify one only, program exited!')
        exit()

    ################################################ get bin to ctg dict ###############################################

    bin_file_re = '%s/*%s' % (bin_folder, bin_ext)
    bin_file_list = [os.path.basename(file_name) for file_name in glob.glob(bin_file_re)]

    if len(bin_file_list) == 0:
        print(datetime.now().strftime(time_format) + 'No bin file found, program exited!')
        exit()

    bin_2_ctg_dict = {}
    for each_bin in bin_file_list:
        pwd_each_bin = '%s/%s' % (bin_folder, each_bin)
        bin_2_ctg_dict[each_bin] = set()
        for seq in SeqIO.parse(pwd_each_bin, 'fasta'):
            bin_2_ctg_dict[each_bin].add(seq.id)

    ############################################ get group to ctg list dict ############################################

    print(datetime.now().strftime(time_format) + 'Get bin (cluster) to contig correlations')

    if bin_cluster_file is None:
        group_2_ctg_dict = bin_2_ctg_dict
    else:
        # get group to bin dict
        group_2_bin_dict = {}
        for group in open(bin_cluster_file):
            group_split = group.strip().split('\t')
            group_2_bin_dict[group_split[0]] = group_split[1:]

        group_2_ctg_dict = {}
        for bin_group in group_2_bin_dict:
            group_member_list = group_2_bin_dict[bin_group]
            group_2_ctg_dict[bin_group] = set()
            for genome_bin in group_member_list:
                genome_bin_ctg_list = bin_2_ctg_dict[genome_bin]
                for ctg in genome_bin_ctg_list:
                    group_2_ctg_dict[bin_group].add(ctg)

    ########################################### get_ref_to_read_num_from_sam ###########################################

    sam_file_path, sam_file_basename, sam_file_extension = sep_path_basename_ext(sam_file)
    ref_to_read_num_file = '%s/%s_ref_to_read_num_%s.txt' % (sam_file_path, sam_file_basename, datetime.now().strftime('%Y-%m-%d_%Hh-%Mm-%Ss_%f'))

    if by_read_len is False:
        print(datetime.now().strftime(time_format) + 'Get the number of reads mapped to each reference sequence in sam file')
        get_ref_to_read_num_from_sam(sam_file, ref_to_read_num_file)
    else:
        print(datetime.now().strftime(time_format) + 'Get the total length of reads mapped to each reference sequence in sam file')
        get_ref_to_total_read_len_from_sam(sam_file, ref_to_read_num_file)

    ########################################### read in ref_to_read_num_file ###########################################

    mapped_reads_num = 0
    ref_to_read_num_dict = {}
    for each_ctg in open(ref_to_read_num_file):
        each_ctg_split = each_ctg.strip().split('\t')
        ctg_id = each_ctg_split[0]
        read_num = int(each_ctg_split[1])
        ref_to_read_num_dict[ctg_id] = read_num
        if ctg_id != '*':
            mapped_reads_num += read_num

    ###################################### get the number of reads in each group #######################################

    print(datetime.now().strftime(time_format) + 'Get the number of reads mapped to each bin (cluster)')

    group_to_read_num_dict = {}
    for group in group_2_ctg_dict:
        group_ctg_list = group_2_ctg_dict[group]
        group_to_read_num_dict[group] = 0
        for ctg in group_ctg_list:
            group_to_read_num_dict[group] += ref_to_read_num_dict.get(ctg, 0)

    output_file_handle = open(output_file, 'w')
    output_file_handle.write('cluster\tread_num\tread_pct\n')
    for group in group_to_read_num_dict:
        group_read_num = group_to_read_num_dict[group]
        group_read_pct = float("{0:.2f}".format(group_read_num*100/mapped_reads_num))
        output_file_handle.write('%s\t%s\t%s\n' % (group, group_read_num, group_read_pct))
    output_file_handle.close()

    ################################################## final report ####################################################

    # delete tmp files
    os.system('rm %s' % ref_to_read_num_file)
    if (cluster_info is None) and (dRep_Cdb_file is not None):
        os.system('rm %s' % bin_cluster_file)

    # final report
    print(datetime.now().strftime(time_format) + 'Done!')


if __name__ == "__main__":

    get_bin_abundance_parser = argparse.ArgumentParser()
    get_bin_abundance_parser.add_argument('-sam',     required=True,                        help='input sam file')
    get_bin_abundance_parser.add_argument('-bin',     required=True,                        help='bin folder')
    get_bin_abundance_parser.add_argument('-x',       required=True, default='fasta',       help='bin file extension, default: fasta')
    get_bin_abundance_parser.add_argument('-o',       required=True,                        help='output abundance file')
    get_bin_abundance_parser.add_argument('-l',       required=False, action="store_true",  help='consider read length')
    get_bin_abundance_parser.add_argument('-g',       required=False, default=None,         help='bin grouping info')
    get_bin_abundance_parser.add_argument('-Cdb',     required=False, default=None,         help='cluster info from dRep (Cdb.csv)')
    args = vars(get_bin_abundance_parser.parse_args())
    get_bin_abundance(args)

'''
cd /Users/songweizhi/Desktop/get_MAG_abundance_wd
python3 /Users/songweizhi/PycharmProjects/BioSAK/BioSAK/magabund.py -sam three_drep_mags.sam -bin mags -x fa -o test -l


'''


import os
import glob
import math
import random
import argparse
import pandas as pd
from Bio import SeqIO
from time import sleep
from Bio.Seq import Seq
import multiprocessing as mp
from functools import reduce


abd_usage = '''
=========================== abd example commands ===========================

BioSAK abd -i sample.txt -o op_dir -t 36 -r reference_masked.fa

# sample.txt file format (tab separated)
Seawater1   path/to/Seawater1_1.fastq.gz    path/to/Seawater1_2.fastq.gz
Seawater2	path/to/Seawater2_1.fastq.gz    path/to/Seawater2_2.fastq.gz
Aphrocallistes  AphrocallistesBeatrix_subset.fastq

# Note
1. Input reads need to be in pair.
2. prepare the genome file with "abd_mask" module

============================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def Reads_simulator(op_dir, op_prefix, pwd_genome_file, read_number, read_depth, read_length, insert_size):

    # read in reference sequences
    ref_seq_total_len = 0
    ref_seq_dict = {}
    ref_seq_len_dict = {}
    for each_seq in SeqIO.parse(pwd_genome_file, 'fasta'):
        seq_id = each_seq.id
        seq_str = str(each_seq.seq)
        ref_seq_dict[seq_id] = seq_str
        ref_seq_len_dict[seq_id] = len(seq_str)
        ref_seq_total_len += len(seq_str)

    # calculate the number of reads to simulate
    if (read_number is None) and (read_depth is not None):
        read_number = round(ref_seq_total_len*read_depth/(read_length*2))
        print('The number of read pairs to simulate: %s' % read_number)

    # get the number of reads to simulate from each ref seq
    ref_to_read_num_dict = {}
    current_ref_index = 1
    total_assigned_read_num = 0
    for each_ref_seq in ref_seq_len_dict:
        if current_ref_index == len(ref_seq_len_dict):
            current_ref_seq_read_num = read_number - total_assigned_read_num
        else:
            current_ref_seq_len_pct = ref_seq_len_dict[each_ref_seq]/ref_seq_total_len
            current_ref_seq_read_num = round(current_ref_seq_len_pct*read_number)
            total_assigned_read_num += current_ref_seq_read_num
        ref_to_read_num_dict[each_ref_seq] = current_ref_seq_read_num
        current_ref_index += 1

    # simulate reads
    output_r1 = '%s/%s_R1.fasta' % (op_dir, op_prefix)
    output_r2 = '%s/%s_R2.fasta' % (op_dir, op_prefix)
    output_r1_handle = open(output_r1, 'w')
    output_r2_handle = open(output_r2, 'w')

    overall_n = 1
    for each_ref_seq in ref_to_read_num_dict:
        ref_seq_str = ref_seq_dict[each_ref_seq]
        ref_seq_read_num = ref_to_read_num_dict[each_ref_seq]
        sequence_length = len(ref_seq_str)
        fragment_length = 2 * read_length + insert_size

        n = 1
        while n <= ref_seq_read_num:
            rdm_num = random.randint(1, sequence_length)
            current_fragment = ''
            if (rdm_num + fragment_length) <= sequence_length:
                current_fragment = ref_seq_str[rdm_num - 1: rdm_num + fragment_length - 1]
            elif (rdm_num + fragment_length) > sequence_length:
                seq_part_1_seq = ref_seq_str[rdm_num - 1:]
                seq_part_2_seq = ref_seq_str[:fragment_length - sequence_length + rdm_num - 1]
                current_fragment = seq_part_1_seq + seq_part_2_seq
            current_fragment_r1 = current_fragment[:read_length]
            current_fragment_r2 = current_fragment[-read_length:]
            current_fragment_r2_reverse_complement = str(Seq(current_fragment_r2).reverse_complement())
            current_read_r1_id = '%s_%s.1' % (op_prefix, overall_n)
            current_read_r2_id = '%s_%s.2' % (op_prefix, overall_n)
            output_r1_handle.write('>%s\n' % current_read_r1_id)
            output_r1_handle.write('%s\n'  % current_fragment_r1)
            output_r2_handle.write('>%s\n' % current_read_r2_id)
            output_r2_handle.write('%s\n'  % current_fragment_r2_reverse_complement)
            n += 1
            overall_n += 1
    output_r1_handle.close()
    output_r2_handle.close()
    print('Simulated reads exported to %s and %s' % (output_r1, output_r2))


def fq2fa(fq_in, fa_out):
    fa_out_handle = open(fa_out, 'w')
    for each_long_read in SeqIO.parse(fq_in, 'fastq'):
        fa_out_handle.write('>%s\n%s\n' % (each_long_read.id, each_long_read.seq))
    fa_out_handle.close()


# paired reads will be simulated from the unpaired long reads
def get_abd2_mapping_worker_long(arg_list):

    op_prefix               = arg_list[0]
    fq_long                 = arg_list[1]
    ref_seq                 = arg_list[2]
    op_dir                  = arg_list[3]
    num_threads             = arg_list[4]
    keep_bam_file           = arg_list[5]
    simulate_num            = arg_list[6]
    simulate_depth          = arg_list[7]
    simulate_len            = arg_list[8]
    simulate_insert_size    = arg_list[9]
    ref_index_dir           = arg_list[10]

    # check input files
    if os.path.isfile(ref_seq) is False:
        print('%s not found, program exited!' % ref_seq)
        exit()

    if os.path.isfile(fq_long) is False:
        print('%s not found, program exited!' % fq_long)
        exit()

    os.system('mkdir %s' % op_dir)

    fq_nonpaired_name, _, _, _ = sep_path_basename_ext(fq_long)
    ref_name, _, _, _ = sep_path_basename_ext(ref_seq)

    perform_decompress              = False
    fq_nonpaired_decompressed       = fq_long
    if fq_long.endswith('.gz'):
        fq_nonpaired_decompressed   = '%s/%s'               % (op_dir, fq_nonpaired_name[:-3])
        gunzip_cmd_long             = 'gunzip -c %s > %s'   % (fq_long, fq_nonpaired_decompressed)
        perform_decompress          = True
        os.system(gunzip_cmd_long)

    # simulate reads here
    long_reads_fasta = '%s/%s_long.fasta' % (op_dir, op_prefix)
    fq2fa(fq_nonpaired_decompressed, long_reads_fasta)
    Reads_simulator(op_dir, op_prefix, long_reads_fasta, simulate_num, simulate_depth, simulate_len, simulate_insert_size)

    r1_fa = '%s/%s_R1.fasta' % (op_dir, op_prefix)
    r2_fa = '%s/%s_R1.fasta' % (op_dir, op_prefix)

    # index reference sequences
    bwa_cmd           = 'bwa mem -5SP -t %s %s/%s %s %s | samblaster > %s/%s.sam'                                                                                           % (num_threads, ref_index_dir, ref_name, r1_fa, r2_fa, op_dir, op_prefix)
    samtools_view_cmd = 'samtools view -@ 32 -bS -h -b %s/%s.sam > %s/%s.bam'                                                                                               % (op_dir, op_prefix, op_dir, op_prefix)
    samtools_sort_cmd = 'samtools sort -@ 32 %s/%s.bam -o %s/%s.sorted.bam'                                                                                                 % (op_dir, op_prefix, op_dir, op_prefix)
    coverm_filter_cmd = 'coverm filter -b %s/%s.sorted.bam --min-read-aligned-percent 0.9 --min-read-percent-identity 0.99 --output-bam-files %s/%s.sorted_filtered.bam'    % (op_dir, op_prefix, op_dir, op_prefix)
    pileup_sh_cmd     = 'pileup.sh in=%s/%s.sorted_filtered.bam out=%s/%s.sorted_filtered.cov rpkm=%s/%s.sorted_filtered.rpkm overwrite=true'                               % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    seqkit_stat_cmd   = 'seqkit stat %s > %s/%s.stat'                                                                                                                       % (r1_fa, op_dir, op_prefix)

    print(seqkit_stat_cmd)
    os.system(seqkit_stat_cmd)

    print(bwa_cmd)
    os.system(bwa_cmd)
    if perform_decompress is True:
        os.system('rm %s/%s' % (op_dir, fq_nonpaired_name[:-3]))

    os.system('rm %s'       % long_reads_fasta)
    os.system('rm %s'       % r1_fa)
    os.system('rm %s'       % r2_fa)
    os.system('rm %s/%s'    % (op_dir, ref_name))
    os.system('rm %s/%s.*'  % (op_dir, ref_name))

    print(samtools_view_cmd)
    os.system(samtools_view_cmd)
    sleep(1)
    os.system('rm %s/%s.sam' % (op_dir, op_prefix))

    print(samtools_sort_cmd)
    os.system(samtools_sort_cmd)
    sleep(1)
    os.system('rm %s/%s.bam' % (op_dir, op_prefix))

    print(coverm_filter_cmd)
    os.system(coverm_filter_cmd)
    sleep(1)
    os.system('rm %s/%s.sorted.bam' % (op_dir, op_prefix))

    print(pileup_sh_cmd)
    os.system(pileup_sh_cmd)
    if keep_bam_file is False:
        os.system('rm %s/%s.sorted_filtered.bam' % (op_dir, op_prefix))
    else:
        os.system('samtools index %s/%s.sorted_filtered.bam' % (op_dir, op_prefix))


def get_abd2_mapping_worker_paired(arg_list):

    op_prefix       = arg_list[0]
    fq_r1           = arg_list[1]
    fq_r2           = arg_list[2]
    ref_seq         = arg_list[3]
    op_dir          = arg_list[4]
    num_threads     = arg_list[5]
    keep_bam_file   = arg_list[6]
    ref_index_dir   = arg_list[7]

    # check input files
    if os.path.isfile(ref_seq) is False:
        print('%s not found, program exited!' % ref_seq)
        exit()

    if os.path.isfile(fq_r1) is False:
        print('%s not found, program exited!' % fq_r1)
        exit()
    if os.path.isfile(fq_r2) is False:
        print('%s not found, program exited!' % fq_r2)
        exit()

    os.system('mkdir %s' % op_dir)

    fq1_name, _, _, _ = sep_path_basename_ext(fq_r1)
    fq2_name, _, _, _ = sep_path_basename_ext(fq_r2)
    ref_name, _, _, _ = sep_path_basename_ext(ref_seq)

    perform_decompress = False

    fq_r1_decompressed = fq_r1
    fq_r2_decompressed = fq_r2
    if fq_r1.endswith('.gz'):
        fq_r1_decompressed  = '%s/%s'               % (op_dir, fq1_name[:-3])
        fq_r2_decompressed  = '%s/%s'               % (op_dir, fq2_name[:-3])
        gunzip_cmd_r1       = 'gunzip -c %s > %s'   % (fq_r1, fq_r1_decompressed)
        gunzip_cmd_r2       = 'gunzip -c %s > %s'   % (fq_r2, fq_r2_decompressed)
        perform_decompress  = True
        os.system(gunzip_cmd_r1)
        os.system(gunzip_cmd_r2)

    # index reference sequences
    bwa_cmd           = 'bwa mem -5SP -t %s %s/%s %s %s | samblaster > %s/%s.sam'                                                                                           % (num_threads, ref_index_dir, ref_name, fq_r1_decompressed, fq_r2_decompressed, op_dir, op_prefix)
    samtools_view_cmd = 'samtools view -@ 32 -bS -h -b %s/%s.sam > %s/%s.bam'                                                                                               % (op_dir, op_prefix, op_dir, op_prefix)
    samtools_sort_cmd = 'samtools sort -@ 32 %s/%s.bam -o %s/%s.sorted.bam'                                                                                                 % (op_dir, op_prefix, op_dir, op_prefix)
    coverm_filter_cmd = 'coverm filter -b %s/%s.sorted.bam --min-read-aligned-percent 0.9 --min-read-percent-identity 0.99 --output-bam-files %s/%s.sorted_filtered.bam'    % (op_dir, op_prefix, op_dir, op_prefix)
    pileup_sh_cmd     = 'pileup.sh in=%s/%s.sorted_filtered.bam out=%s/%s.sorted_filtered.cov rpkm=%s/%s.sorted_filtered.rpkm overwrite=true'                               % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    seqkit_stat_cmd   = 'seqkit stat %s > %s/%s.stat'                                                                                                                       % (fq_r1_decompressed, op_dir, op_prefix)

    print(seqkit_stat_cmd)
    os.system(seqkit_stat_cmd)

    print(bwa_cmd)
    os.system(bwa_cmd)
    if perform_decompress is True:
        os.system('rm %s/%s' % (op_dir, fq1_name[:-3]))
        os.system('rm %s/%s' % (op_dir, fq2_name[:-3]))
    os.system('rm %s/%s'   % (op_dir, ref_name))
    os.system('rm %s/%s.*' % (op_dir, ref_name))

    print(samtools_view_cmd)
    os.system(samtools_view_cmd)
    sleep(1)
    os.system('rm %s/%s.sam' % (op_dir, op_prefix))

    print(samtools_sort_cmd)
    os.system(samtools_sort_cmd)
    sleep(1)
    os.system('rm %s/%s.bam' % (op_dir, op_prefix))

    print(coverm_filter_cmd)
    os.system(coverm_filter_cmd)
    sleep(1)
    os.system('rm %s/%s.sorted.bam' % (op_dir, op_prefix))

    print(pileup_sh_cmd)
    os.system(pileup_sh_cmd)
    if keep_bam_file is False:
        os.system('rm %s/%s.sorted_filtered.bam' % (op_dir, op_prefix))
    else:
        os.system('samtools index %s/%s.sorted_filtered.bam' % (op_dir, op_prefix))


def get_abd3_stats(rpkm_stat_cov_dir, genome_rename_file, stats_dir):

    # check input files
    if os.path.isdir(rpkm_stat_cov_dir) is False:
        print('%s not found, program exited!' % rpkm_stat_cov_dir)
        exit()

    rpkm_file_re    = '%s/*.rpkm' % rpkm_stat_cov_dir
    stat_file_re    = '%s/*.stat' % rpkm_stat_cov_dir
    rpkm_file_list  = glob.glob(rpkm_file_re)
    stat_file_list  = glob.glob(stat_file_re)

    if len(rpkm_file_list) == 0:
        print('.rpkm file not found, program exited!')
        exit()

    if len(stat_file_list) == 0:
        print('.stat file not found, program exited!')
        exit()

    # define output file name
    gnm_level_rpkm_dir               = '%s/rpkm_by_genome'              % stats_dir
    combined_rpkm_file               = '%s/rpkm_df.txt'                 % stats_dir
    combined_rpkm_file_renamed       = '%s/rpkm_df.renamed.txt'         % stats_dir
    combined_rpkm_file_log10         = '%s/rpkm_df.log10.txt'           % stats_dir
    combined_rpkm_file_log10_renamed = '%s/rpkm_df.renamed.log10.txt'   % stats_dir

    # create op dir
    os.system('mkdir %s' % stats_dir)
    os.system('mkdir %s' % gnm_level_rpkm_dir)

    # get genome rename dict
    gnm_rename_dict = dict()
    if genome_rename_file is not None:
        for line in open(genome_rename_file):
            line_split = line.strip().split('\t')
            gnm_rename_dict[line_split[0]] = line_split[1]

    # read in stat file
    sample_total_read_num_dict = dict()
    for stat_file in sorted(stat_file_list):
        _, _, stat_base, _ = sep_path_basename_ext(stat_file)
        stat_file_lines = open(stat_file).readlines()
        info_line_split = stat_file_lines[-1].strip().split(' ')
        while '' in info_line_split:
            info_line_split.remove('')
        read_num = int(info_line_split[3].replace(',', ''))
        sample_total_read_num_dict[stat_base] = read_num

    # summarise rpkm on genome level
    sample_id_list = []
    for rpkm_file in sorted(rpkm_file_list):
        rpkm_name, rpkm_path, rpkm_base, rpkm_ext = sep_path_basename_ext(rpkm_file)
        sample_id = rpkm_base.split('.sorted_filtered')[0]
        sample_id_list.append(sample_id)
        op_txt = '%s/%s.cal_bin.rpkm' % (gnm_level_rpkm_dir, sample_id)
        rpkm_df = pd.read_csv(rpkm_file, sep='\t', header=4)
        subset_df_col_list = ['#Name', 'Length', 'Reads']
        sample_total_read_num = sample_total_read_num_dict[sample_id]
        subset_df = rpkm_df.loc[:, subset_df_col_list]
        subset_df['bin_name'] = subset_df['#Name'].str.rsplit('_', n=1, expand=True)[0]
        subset_df['Length'] = subset_df['Length'].astype(int)
        subset_df['Reads'] = subset_df['Reads'].astype(int)

        # calulate RPKM
        op_df = subset_df.groupby(["bin_name"])[['Length', 'Reads']].sum()
        op_df[sample_id] = (op_df['Reads']*1000000000)/(op_df['Length']*sample_total_read_num*2)
        op_df.to_csv(op_txt, sep='\t')

    # combine the results
    rpkm_df_list = []
    for sample_id in sample_id_list:
        gnm_level_rpkm_file = '%s/%s.cal_bin.rpkm' % (gnm_level_rpkm_dir, sample_id)
        gnm_level_rpkm_df = pd.read_csv(gnm_level_rpkm_file, sep='\t')
        gnm_level_rpkm_df = gnm_level_rpkm_df.drop(["Length", "Reads"], axis=1)
        rpkm_df_list.append(gnm_level_rpkm_df)

    # write out combined_rpkm_file
    rpkm_df_combined = reduce(lambda left, right: pd.merge(left, right, on='bin_name',how='outer'), rpkm_df_list)
    rpkm_df_combined.to_csv(combined_rpkm_file, sep='\t',index=False)

    # write out combined_rpkm_file_log10
    combined_rpkm_file_log10_handle = open(combined_rpkm_file_log10, 'w')
    for each_line in open(combined_rpkm_file):
        each_line_split = each_line.strip().split('\t')
        if each_line.startswith('bin_name'):
            combined_rpkm_file_log10_handle.write(each_line)
        else:
            value_list = [each_line_split[0]]
            for each_value in each_line_split[1:]:
                each_value = float(each_value)
                if each_value == 0:
                    value_list.append('na')
                else:
                    each_value_log = math.log(each_value)
                    value_list.append(each_value_log)
            combined_rpkm_file_log10_handle.write('%s\n' % '\t'.join([str(i) for i in value_list]))
    combined_rpkm_file_log10_handle.close()

    # rename gnm id in the final dataframe
    if len(gnm_rename_dict) > 0:

        # rename combined_rpkm_file_renamed
        combined_rpkm_file_renamed_handle = open(combined_rpkm_file_renamed, 'w')
        for each_line in open(combined_rpkm_file):
            if each_line.startswith('bin_name'):
                combined_rpkm_file_renamed_handle.write(each_line)
            else:
                each_line_split = each_line.strip().split('\t')
                gnm_id = each_line_split[0]
                gnm_id_new = gnm_rename_dict.get(gnm_id, gnm_id)
                combined_rpkm_file_renamed_handle.write(each_line.replace(gnm_id, gnm_id_new))
        combined_rpkm_file_renamed_handle.close()

        # rename combined_rpkm_file_log10_renamed
        combined_rpkm_file_log10_renamed_handle = open(combined_rpkm_file_log10_renamed, 'w')
        for each_line in open(combined_rpkm_file_log10):
            if each_line.startswith('bin_name'):
                combined_rpkm_file_log10_renamed_handle.write(each_line)
            else:
                each_line_split = each_line.strip().split('\t')
                gnm_id = each_line_split[0]
                gnm_id_new = gnm_rename_dict.get(gnm_id, gnm_id)
                combined_rpkm_file_log10_renamed_handle.write(each_line.replace(gnm_id, gnm_id_new))
        combined_rpkm_file_log10_renamed_handle.close()


def abd(args):

    input_txt               = args['i']
    ref_seq                 = args['r']
    op_dir                  = args['o']
    keep_bam_file           = args['keep_bam']
    num_threads             = args['t']
    force_overwrite         = args['f']
    rename_txt              = args['rename']

    # parameters for read_simulator
    read_simulator_num          = 5000000
    read_simulator_depth        = None
    read_simulator_len          = 150
    read_simulator_insert_size  = 200

    # check input file
    if rename_txt is not None:
        if os.path.isfile(rename_txt) is False:
            print('%s not found, program exited!' % rename_txt)
            exit()

    # define file name
    ref_seq_index_dir   = '%s/ref_seq_index'        % op_dir
    mapping_wd          = '%s/mapping_wd'           % op_dir
    cov_rpkm_stat_dir   = '%s/cov_rpkm_stat_files'  % op_dir
    stats_dir           = '%s/stats_dir'            % op_dir

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    #################### index reference sequences ####################

    os.system('mkdir %s' % ref_seq_index_dir)

    ref_name, _, _, _ = sep_path_basename_ext(ref_seq)
    index_ref_cmd     = 'cp %s %s/; bwa index %s/%s'    % (ref_seq, ref_seq_index_dir, ref_seq_index_dir, ref_name)
    print(index_ref_cmd)
    os.system(index_ref_cmd)

    #################### mapping ####################

    os.system('mkdir %s' % mapping_wd)

    # run mapping
    sample_lol_long = []
    sample_lol_paired = []
    for each_sample in open(input_txt):
        current_arg_list = each_sample.strip().split('\t')
        if len(current_arg_list) == 3:
            sample_lol_paired.append(current_arg_list)
        elif len(current_arg_list) == 2:
            sample_lol_long.append(current_arg_list)

    #################### mapping paired reads ####################

    if len(sample_lol_paired) > 0:
        threads_per_job_paired = num_threads//len(sample_lol_paired)

        arg_lol= []
        for arg_list in sample_lol_paired:
            sample_id = arg_list[0]
            current_op_dir = '%s/%s' % (mapping_wd, sample_id)
            arg_list.append(ref_seq)
            arg_list.append(current_op_dir)
            arg_list.append(threads_per_job_paired)
            arg_list.append(keep_bam_file)
            arg_list.append(ref_seq_index_dir)
            arg_list.append(mapping_wd)
            arg_lol.append(arg_list)

        # run mapping with multiprocessing
        pool = mp.Pool(processes=num_threads)
        pool.map(get_abd2_mapping_worker_paired, arg_lol)
        pool.close()
        pool.join()

    #################### mapping long reads ####################

    if len(sample_lol_long) > 0:
        threads_per_job_long = num_threads//len(sample_lol_long)

        arg_lol= []
        for arg_list in sample_lol_long:
            sample_id = arg_list[0]
            current_op_dir = '%s/%s' % (mapping_wd, sample_id)
            arg_list.append(ref_seq)
            arg_list.append(current_op_dir)
            arg_list.append(threads_per_job_long)
            arg_list.append(keep_bam_file)
            arg_list.append(read_simulator_num)
            arg_list.append(read_simulator_depth)
            arg_list.append(read_simulator_len)
            arg_list.append(read_simulator_insert_size)
            arg_list.append(ref_seq_index_dir)
            arg_list.append(mapping_wd)
            arg_lol.append(arg_list)

        # run mapping with multiprocessing
        pool = mp.Pool(processes=num_threads)
        pool.map(get_abd2_mapping_worker_long, arg_lol)
        pool.close()
        pool.join()

    #################### get stats ####################

    # mkdir
    os.mkdir(cov_rpkm_stat_dir)
    cp_cmd_rpkm = 'cp %s/*/*.rpkm %s'   % (mapping_wd, cov_rpkm_stat_dir)
    cp_cmd_stat = 'cp %s/*/*.stat %s'   % (mapping_wd, cov_rpkm_stat_dir)
    cp_cmd_cov  = 'cp %s/*/*.cov %s'    % (mapping_wd, cov_rpkm_stat_dir)
    os.system(cp_cmd_rpkm)
    os.system(cp_cmd_stat)
    os.system(cp_cmd_cov)

    # get stats
    get_abd3_stats(cov_rpkm_stat_dir, rename_txt, stats_dir)

    #################### final report ####################

    # final report
    print('Estimated abundance exported to:')
    print('%s/rpkm_df.txt'       % stats_dir)
    print('%s/rpkm_df.log10.txt' % stats_dir)
    print('Done!')


if __name__ == '__main__':

    abd_parser = argparse.ArgumentParser()
    abd_parser.add_argument('-i',           required=True,                          help='input txt')
    abd_parser.add_argument('-r',           required=True,                          help='reference, need to be masked')
    abd_parser.add_argument('-o',           required=True,                          help='output directory')
    abd_parser.add_argument('-rename',      required=False, default=None,           help='rename genome id in the final dataframe')
    abd_parser.add_argument('-keep_bam',    required=False, action="store_true",    help='do not delete bam file')
    abd_parser.add_argument('-t',           required=False, type=int, default=1,    help='number of threads, default is 1')
    abd_parser.add_argument('-f',           required=False, action="store_true",    help='force overwrite')
    args = vars(abd_parser.parse_args())
    abd(args)





SRA_Biosample_dict = dict()
SRA_Bioproject_dict = dict()
SRA_Biosample_Bioproject_txt = '/Users/songweizhi/Desktop/SRA_Biosample_Bioproject.txt'
for each_line in open(SRA_Biosample_Bioproject_txt):
    each_split = each_line.strip().split('\t')
    SRA_Biosample_dict[each_split[0]] = each_split[1]
    SRA_Bioproject_dict[each_split[0]] = each_split[2]

metadata_dict = dict()
metadata_txt = '/Users/songweizhi/Desktop/biosamples_uniq/metadata.txt'
for each_line in open(metadata_txt):
    each_split = each_line.strip().split('\t')
    metadata_dict[each_split[0]] = each_line.strip()




op_txt = '/Users/songweizhi/Desktop/op.txt'
op_txt_handle = open(op_txt, 'w')

for each in open("/Users/songweizhi/Desktop/150_MAGs.txt"):
    each_split = each.strip().split("\t")
    op_str = ''
    if ('SRR' in each_split[1]) or ('ERR' in each_split[1]):
        current_biosample = SRA_Biosample_dict.get(each_split[1], 'na')
        current_bioproject = SRA_Bioproject_dict.get(each_split[1], 'na')
        op_str = '%s\t%s\t%s\t%s\t%s' % (each_split[0], each_split[1], current_biosample, current_bioproject, metadata_dict.get(current_biosample, 'na'))
    else:
        op_str = '%s\t%s\t%s\t%s\t%s' % (each_split[0], 'na', each_split[1], 'na', metadata_dict.get(each_split[1], 'na'))
    op_txt_handle.write(op_str + '\n')
    print(op_str)
op_txt_handle.close()


import os
import glob
import argparse
from Bio import SeqIO


cat_fa_usage = '''
================= cat_fa example commands =================

# example command 
BioSAK cat_fa -i seq_dir -x fa -o combined_seqs.fa

# combine sequence files in a folder after prefixing
# sequence ids with corresponding file name.

===========================================================
'''


def cat_fa(args):

    seq_folder    = args['i']
    file_ext      = args['x']
    sep_str       = args['s']
    combined_file = args['o']

    seq_file_re = '%s/*.%s' % (seq_folder, file_ext)
    seq_file_list = [os.path.basename(file_name) for file_name in glob.glob(seq_file_re)]

    combined_file_handle = open(combined_file, 'w')
    for seq_file in seq_file_list:
        pwd_seq_file = '%s/%s' % (seq_folder, seq_file)
        file_name_no_ext = seq_file[:-(len(file_ext)+1)]
        for each_seq in SeqIO.parse(pwd_seq_file, 'fasta'):
            seq_name_new = '%s%s%s' % (file_name_no_ext, sep_str, each_seq.id)
            each_seq.id = seq_name_new
            SeqIO.write(each_seq, combined_file_handle, 'fasta')
    combined_file_handle.close()


if __name__ == "__main__":

    cat_fa_parser = argparse.ArgumentParser()
    cat_fa_parser.add_argument('-i', required=True,                     help='sequence folder')
    cat_fa_parser.add_argument('-x', required=False, default='fasta',   help='sequence file extension, default: fasta')
    cat_fa_parser.add_argument('-s', required=False, default='__',      help='separator, default: __')
    cat_fa_parser.add_argument('-o', required=True,                     help='combined output file')
    args = vars(cat_fa_parser.parse_args())
    cat_fa(args)



import argparse
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt


vis_color_scheme_usage = '''
=================== vis_color_scheme example commands ===================

BioSAK vis_color_scheme -i lineage.txt -c tax_color.txt -o output.pdf
BioSAK vis_color_scheme -i Sponge_full_lineage_GTDB_format.txt -c color_code_sponge.txt -o color_code_sponge.pdf

=========================================================================
'''


def vis_color_scheme(args):

    tax_full_lineage_txt = args['i']
    tax_to_color_txt     = args['c']
    plot_width           = args['x']
    plot_height          = args['y']
    output_plot          = args['o']

    # read in taxon to color info
    taxon_to_color_dict = dict()
    for each_taxon in open(tax_to_color_txt):
        each_taxon_split = each_taxon.strip().split('\t')
        taxon_to_color_dict[each_taxon_split[0]] = each_taxon_split[1]

    max_lineage_len = 0
    full_lineage_list = []
    max_label_len = 0
    for each_lineage in open(tax_full_lineage_txt):
        each_lineage_split = each_lineage.strip().split(';')
        if len(each_lineage_split) > max_lineage_len:
                max_lineage_len = len(each_lineage_split)
        full_lineage_list.append(each_lineage_split)
        for each_rank in each_lineage_split:
                if len(each_rank) > max_label_len:
                    max_label_len = len(each_rank)

    fig = plt.figure(1, figsize=(plot_width, plot_height))
    ax = fig.add_subplot(111)

    y_pos = 0.97
    for each_lineage in full_lineage_list:
        x_pos = 0.005
        for each_rank in each_lineage:
            current_color = taxon_to_color_dict.get(each_rank, 'white')

            if current_color == 'white':
                label_txt = each_rank
                label_txt += ' '*((max_label_len - len(each_rank)) + 10)
            else:
                extra_space_str = ' '*(max_label_len - len(each_rank))
                label_txt = '%s %s%s' % (each_rank, extra_space_str, current_color)

            if current_color == 'white':
                ax.text(x_pos, y_pos, label_txt, color='black', family='monospace', bbox=dict(facecolor=current_color, edgecolor='black'))
            else:
                ax.text(x_pos, y_pos, label_txt, color='black', family='monospace', bbox=dict(facecolor=current_color, edgecolor=current_color))

            x_pos += (1/max_lineage_len)
        y_pos -= (1/(len(full_lineage_list)+2))

    # export plot
    plt.margins(tight=True)
    plt.savefig(output_plot, dpi=1000)
    plt.close()


if __name__ == '__main__':

    vis_color_scheme_parser = argparse.ArgumentParser(usage=vis_color_scheme_usage)
    vis_color_scheme_parser.add_argument('-i',  required=True,                          help='lineage file')
    vis_color_scheme_parser.add_argument('-c',  required=True,                          help='taxon color file')
    vis_color_scheme_parser.add_argument('-x',  required=False,type=int, default=42,    help='plot width, default is 42')
    vis_color_scheme_parser.add_argument('-y',  required=False,type=int, default=22,    help='plot height, default is 22')
    vis_color_scheme_parser.add_argument('-o',  required=True,                          help='output pdf')
    args = vars(vis_color_scheme_parser.parse_args())
    vis_color_scheme(args)



import os
import argparse
from Bio import SeqIO


fq2fa_usage = '''
========== fq2fa example commands ==========

BioSAK fq2fa -i reads_R1.fq -o reads_R1.fa
BioSAK fq2fa -i reads_R2.fq -o reads_R2.fa

============================================
'''

def fq2fa(args):

    fq_in   = args['i']
    fa_out  = args['o']

    fa_out_handle = open(fa_out, 'w')
    for each_long_read in SeqIO.parse(fq_in, 'fastq'):
        fa_out_handle.write('>%s\n%s\n' % (each_long_read.id, each_long_read.seq))
    fa_out_handle.close()


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument('-i', required=True, help='input fastq')
    parser.add_argument('-o', required=True, help='output fasta')
    args = vars(parser.parse_args())
    fq2fa(args)



import argparse


count_num_usage = '''
==== count_num example command ====

BioSAK count_num -i genus.txt

===================================
'''

def count_num(args):

    file_in = args['i']

    count_num_dict = dict()
    for each_line in open(file_in):
        each_line_split = each_line.strip().split()
        if each_line_split[0] not in count_num_dict:
            count_num_dict[each_line_split[0]] = 1
        else:
            count_num_dict[each_line_split[0]] += 1

    for each_key in sorted(list(count_num_dict.keys())):
        print('%s\t%s' % (each_key, count_num_dict[each_key]))


if __name__ == '__main__':

    parser = argparse.ArgumentParser(usage=count_num_usage)
    parser.add_argument('-i', required=True, help='input file')
    args = vars(parser.parse_args())
    count_num(args)



import os
import glob
import argparse


Combine_KEGG_COG_usage = '''
================= Combine_KEGG_COG example commands =================

BioSAK Combine_KEGG_COG -kegg KEGG_wd -arcog arCOG_wd -o op_dir -f

=====================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def Combine_KEGG_COG(args):

    output_folder       = args['o']
    kegg_annotation_wd  = args['kegg']
    cog_annotation_wd   = args['cog']
    force_create_op_dir = args['f']

    ############################################## define output file name #############################################

    fun_assignment_dir  = '%s/fun_assignment'   % output_folder
    stats_dir_num       = '%s/fun_stats_num'    % output_folder
    stats_dir_pct       = '%s/fun_stats_pct'    % output_folder

    # create output folder
    if os.path.isdir(output_folder) is True:
        if force_create_op_dir is True:
            os.system('rm -r %s' % output_folder)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % output_folder)
    os.system('mkdir %s' % fun_assignment_dir)
    os.system('mkdir %s' % stats_dir_num)
    os.system('mkdir %s' % stats_dir_pct)

    ################################################### get file list ##################################################

    cog_assignment_re        = '%s/*_*COG*_wd/*_query_to_cog.txt'      % cog_annotation_wd
    ko_assignment_re         = '%s/*_KEGG_wd/*_ko_assignment_ABCD.txt' % kegg_annotation_wd
    cog_assignment_file_list = glob.glob(cog_assignment_re)
    ko_assignment_file_list  = glob.glob(ko_assignment_re)

    ######################################### read in kegg annotation results ##########################################

    fun_id_to_description_dict = dict()
    fun_id_to_B_dict = dict()
    fun_id_to_C_dict = dict()
    gnm_to_gene_dict = dict()
    ko_assignment_dod = dict()
    for each_file in ko_assignment_file_list:
        f_name, f_path, f_base, f_ext = sep_path_basename_ext(each_file)
        gnm_id = f_base[:-len('_ko_assignment_ABCD')]

        if gnm_id not in gnm_to_gene_dict:
            gnm_to_gene_dict[gnm_id] = set()

        current_genome_ko_assignment_dict = dict()
        for each_line in open(each_file):
            if not each_line.startswith('Gene_id\t'):
                each_line_split = each_line.strip().split('\t')
                gene_id = each_line_split[0]
                gnm_to_gene_dict[gnm_id].add(gene_id)
                if len(each_line_split) > 1:
                    ko_id       = each_line_split[4][2:]
                    ko_b        = each_line_split[2]
                    ko_b_desc   = each_line_split[6]
                    ko_c        = each_line_split[3]
                    ko_c_desc   = each_line_split[7]
                    ko_desc     = each_line_split[8].strip()
                    ko_b_with_desc = '%s__%s' % (ko_b, ko_b_desc)
                    ko_c_with_desc = '%s__%s' % (ko_c, ko_c_desc)
                    fun_id_to_B_dict[ko_id] = ko_b_with_desc
                    fun_id_to_C_dict[ko_id] = ko_c_with_desc
                    fun_id_to_description_dict[ko_id] = ko_desc
                    current_genome_ko_assignment_dict[gene_id] = ko_id
        ko_assignment_dod[gnm_id] = current_genome_ko_assignment_dict

    ######################################### read in arcog annotation results #########################################

    cog_assignment_dod = dict()
    cog_id_to_cate_dict = dict()
    cog_cate_to_desc_dict = dict()
    for each_file in cog_assignment_file_list:

        f_name, f_path, f_base, f_ext = sep_path_basename_ext(each_file)
        gnm_id = f_base[:-len('_query_to_cog')]

        if gnm_id not in gnm_to_gene_dict:
            gnm_to_gene_dict[gnm_id] = set()

        current_genome_arcog_assignment_dict = dict()
        for each_line in open(each_file):
            if not each_line.startswith('Query\tarCOG'):
                each_line_split = each_line.strip().split('\t')
                gene_id = each_line_split[0]
                gnm_to_gene_dict[gnm_id].add(gene_id)
                if len(each_line_split) > 1:
                    cog_id = each_line_split[1]
                    cog_cates = each_line_split[2]
                    cog_desc = each_line_split[3]

                    if cog_id not in cog_id_to_cate_dict:
                        cog_id_to_cate_dict[cog_id] = set()

                    for arcog_cate in cog_cates:
                        cog_id_to_cate_dict[cog_id].add(arcog_cate)

                    fun_id_to_description_dict[cog_id] = cog_desc
                    current_genome_arcog_assignment_dict[gene_id] = cog_id
        cog_assignment_dod[gnm_id] = current_genome_arcog_assignment_dict

        # read in cog cate to desc info
        func_stats_txt = each_file.replace('_query_to_cog.txt', '_func_stats.txt')
        for each in open(func_stats_txt):
            each_split = each.strip().split('\t')
            cate_id = each_split[0]
            cate_desc = each_split[2]
            cog_cate_to_desc_dict[cate_id] = cate_desc

    ###################################### write out combined annotation results #######################################

    # write out combined annotation results
    combined_annotation_dod = dict()
    for each_gnm in sorted(list(gnm_to_gene_dict.keys())):
        gnm_arcog_dict = cog_assignment_dod[each_gnm]
        gnm_kegg_dict  = ko_assignment_dod[each_gnm]
        gnm_gene_list  = sorted(list(gnm_to_gene_dict[each_gnm]))
        combined_annotation_dict = dict()
        combined_annotation_txt = '%s/%s.txt' % (fun_assignment_dir, each_gnm)
        combined_annotation_txt_handle = open(combined_annotation_txt, 'w')
        for each_gene in gnm_gene_list:
            if each_gene in gnm_kegg_dict:
                fun_assignment = gnm_kegg_dict[each_gene]
                fun_desc = fun_id_to_description_dict[fun_assignment]
                combined_annotation_txt_handle.write('%s\t%s\t%s\n' % (each_gene, fun_assignment, fun_desc))
                combined_annotation_dict[each_gene] = gnm_kegg_dict[each_gene]
            elif each_gene in gnm_arcog_dict:
                fun_assignment = gnm_arcog_dict[each_gene]
                fun_desc = fun_id_to_description_dict[fun_assignment]
                combined_annotation_txt_handle.write('%s\t%s\t%s\n' % (each_gene, fun_assignment, fun_desc))
                combined_annotation_dict[each_gene] = gnm_arcog_dict[each_gene]
            else:
                combined_annotation_txt_handle.write(each_gene + '\n')
        combined_annotation_txt_handle.close()

        combined_annotation_dod[each_gnm] = combined_annotation_dict

    ############################################ write out annotation stats ############################################

    for each_gnm in sorted(list(gnm_to_gene_dict.keys())):
        annotation_dict = combined_annotation_dod[each_gnm]

        fun_stats_num_txt = '%s/%s.txt' % (stats_dir_num, each_gnm)
        fun_stats_pct_txt = '%s/%s.txt' % (stats_dir_pct, each_gnm)

        stats_dict = dict()
        gene_num = 0
        for each_gene in annotation_dict:
            fun_id = annotation_dict[each_gene]
            if fun_id not in stats_dict:
                stats_dict[fun_id] = 1
            else:
                stats_dict[fun_id] += 1
            gene_num += 1

        stats_dict_pct = dict()
        for each_fun in stats_dict:
            fun_num = stats_dict[each_fun]
            fun_pct = fun_num*100/gene_num
            fun_pct = float("{0:.2f}".format(fun_pct))
            stats_dict_pct[each_fun] = fun_pct

        fun_stats_num_txt_handle = open(fun_stats_num_txt, 'w')
        fun_stats_pct_txt_handle = open(fun_stats_pct_txt, 'w')
        for each_fun in sorted(list(stats_dict.keys())):
            fun_num = stats_dict[each_fun]
            fun_pct = stats_dict_pct[each_fun]
            fun_desc = fun_id_to_description_dict[each_fun]

            fun_note = ''
            if each_fun.startswith('arCOG'):
                fun_cate_list = sorted(list(cog_id_to_cate_dict[each_fun]))
                for fun_cate in fun_cate_list:
                    fun_cate_desc = cog_cate_to_desc_dict[fun_cate]
                    fun_note += ('%s__%s' % (fun_cate, fun_cate_desc))
            elif each_fun.startswith('K'):
                fun_b = fun_id_to_B_dict[each_fun]
                fun_c = fun_id_to_C_dict[each_fun]
                fun_note = '%s;%s' % (fun_c, fun_b)
            fun_stats_num_txt_handle.write('%s\t%s\t%s\t%s\n' % (each_fun, fun_num, fun_desc, fun_note))
            fun_stats_pct_txt_handle.write('%s\t%s\t%s\t%s\n' % (each_fun, fun_pct, fun_desc, fun_note))
        fun_stats_num_txt_handle.close()
        fun_stats_pct_txt_handle.close()

    ####################################################################################################################


if __name__ == '__main__':

    Combine_KEGG_COG_parser = argparse.ArgumentParser(usage=Combine_KEGG_COG_usage)
    Combine_KEGG_COG_parser.add_argument('-o',        required=True,                          help='output directory')
    Combine_KEGG_COG_parser.add_argument('-kegg',     required=True,                          help='annotation results from KEGG module')
    Combine_KEGG_COG_parser.add_argument('-cog',      required=True,                          help='annotation results from COG module')
    Combine_KEGG_COG_parser.add_argument('-f',        required=False, action="store_true",    help='force overwrite')
    args = vars(Combine_KEGG_COG_parser.parse_args())
    Combine_KEGG_COG(args)



import os
import glob


def summarise_metadata(file_dir, file_ext, summary_txt):


    file_re = '%s/*.%s' % (file_dir, file_ext)
    file_list = glob.glob(file_re)

    for each_file in file_list:
        file_name = os.path.basename(each_file)
        sample_id = file_name[:-(len(file_ext) + 1)]
        current_isolate = 'na'
        current_host = 'na'
        for each_line in open(each_file):
            if '/host="' in each_line:
                current_host = each_line.split('"')[1]
            elif '/isolate="' in each_line:
                current_isolate = each_line.split('"')[1]

        print('%s\t%s\t%s' % (sample_id, current_isolate, current_host))



file_dir    = '/Users/songweizhi/Desktop/biosamples_metadata/tmp'
file_ext    = 'txt'
summary_txt = '/Users/songweizhi/Desktop/biosamples_metadata/tmp.txt'


summarise_metadata(file_dir, file_ext, summary_txt)


import os
import argparse


mmseqs_usage = '''
============================== mmseqs example command ==============================

BioSAK mmseqs -i contigs.fa -o op_dir -db /scratch/PI/ocessongwz/DB/mmseqs/GTDB -f

====================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def mmseqs(args):

    seq_file        = args['i']
    op_dir          = args['o']
    db_dir          = args['db']
    num_threads     = args['t']
    force_overwrite = args['f']

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    _, _, seq_file_base, _ = sep_path_basename_ext(seq_file)

    cmd_1 = 'mmseqs createdb %s %s/%s.DB'                                                               % (seq_file, op_dir, seq_file_base)
    cmd_2 = 'mmseqs taxonomy %s/%s.DB %s %s/taxonomyResult %s/tmp --threads %s'                         % (op_dir, seq_file_base, db_dir, op_dir, op_dir, num_threads)
    cmd_3 = 'mmseqs createtsv %s/%s.DB %s/taxonomyResult %s/taxonomyResult.tsv'                         % (op_dir, seq_file_base, op_dir, op_dir)
    cmd_4 = 'mmseqs taxonomyreport %s %s/taxonomyResult %s/taxonomyResult_report'                       % (db_dir, op_dir, op_dir)
    cmd_5 = 'mmseqs taxonomyreport %s %s/taxonomyResult %s/taxonomyResult_report.html --report-mode 1'  % (db_dir, op_dir, op_dir)

    os.system(cmd_1)
    os.system(cmd_2)
    os.system(cmd_3)
    os.system(cmd_4)
    os.system(cmd_5)

    print('Classification results exported to:')
    print('Kraken format\t%s/taxonomyResult_report'     % op_dir)
    print('Krona format\t%s/taxonomyResult_report.html' % op_dir)
    print('For the visualization of the Kraken report using Pavian: https://github.com/soedinglab/mmseqs2/wiki#taxonomy-assignment')


if __name__ == '__main__':

    mmseqs_parser = argparse.ArgumentParser(usage=mmseqs_usage)
    mmseqs_parser.add_argument('-i',    required=True,                          help='sequence file')
    mmseqs_parser.add_argument('-o',    required=True,                          help='output directory')
    mmseqs_parser.add_argument('-db',   required=True,                          help='db dir')
    mmseqs_parser.add_argument('-t',    required=False, type=int, default=1,    help='number of threads, default is 1')
    mmseqs_parser.add_argument('-f',    required=False, action="store_true",    help='force overwrite')
    args = vars(mmseqs_parser.parse_args())
    mmseqs(args)



import os
import argparse
from Bio import SeqIO
from reportlab.lib import colors
from reportlab.lib.units import cm
from Bio.Graphics import GenomeDiagram
from Bio.Graphics.GenomeDiagram import CrossLink

cross_link_seqs_usage = '''
==================== cross_link_seqs example commands ====================

module load blast+
BioSAK cross_link_seqs -p Demo -1 ctg1.fa -2 ctg2.fa -l 500 -i 99.9 
BioSAK cross_link_seqs -p Demo -1 ctg1.fa -2 ctg2.fa -l 500 -i 95 -f SVG

==========================================================================
'''


def cross_link_seqs(args):

    prefix          = args['p']
    query_c         = args['1']
    subject_c       = args['2']
    iden_cutoff     = args['i']
    aln_len_cutoff  = args['l']
    plot_format     = args['f']

    # check plot_format
    if plot_format not in ['PDF', 'SVG', 'EPS', 'PNG']:
        print('please choose plot format from PDF, SVG, EPS and PNG')
        exit()

    blast_result_txt          = '%s_blastn.txt'                 % prefix
    blast_result_txt_filtered = '%s_blastn_iden%s_aln%sbp.txt'  % (prefix, iden_cutoff, aln_len_cutoff)
    output_plot               = '%s_iden%s_aln%sbp.%s'          % (prefix, iden_cutoff, aln_len_cutoff, plot_format)

    ############################## prepare for flanking plot ##############################

    # Run Blast
    command_blast = 'blastn -query %s -subject %s -out %s -evalue 1e-5 -outfmt 6 -task blastn' % (query_c, subject_c, blast_result_txt)
    os.system(command_blast)

    # filter blast result
    aln_len_set = set()
    iden_set = set()
    blast_result_txt_filtered_handle = open(blast_result_txt_filtered, 'w')
    for blast_hit in open(blast_result_txt):
        blast_hit_split = blast_hit.strip().split('\t')
        iden = float(blast_hit_split[2])
        align_len = int(blast_hit_split[3])
        if (iden >= iden_cutoff) and (align_len >= aln_len_cutoff):
            blast_result_txt_filtered_handle.write(blast_hit)
            iden_set.add(iden)
            aln_len_set.add(align_len)
    blast_result_txt_filtered_handle.close()

    if len(aln_len_set) == 0:
        print('No matched region was found with specified identity and alignment length cut-offs, program exited!')
        exit()

    iden_min    = min([i for i in iden_set])
    iden_max    = max([i for i in iden_set])
    aln_len_min = min([i for i in aln_len_set])
    aln_len_max = max([i for i in aln_len_set])

    # read in sequences
    ctg_1_record = SeqIO.read(query_c, "fasta")
    ctg_2_record = SeqIO.read(subject_c, "fasta")

    # create an empty diagram and set track length
    diagram = GenomeDiagram.Diagram("TITLE HERE")
    max_len = max(len(ctg_1_record), len(ctg_2_record))

    # add gene content track for gene1_contig
    ctg_1_name_str = '%s (%sbp)    Identity (%s-%s%s)    Alignment length (%s-%sbp)' % (ctg_1_record.id, len(ctg_1_record), iden_min, iden_max, '%', aln_len_min, aln_len_max)
    contig_1_gene_content_track = diagram.new_track(1, name=ctg_1_name_str, greytrack=True, greytrack_labels=1,
                                                    greytrack_font='Helvetica', greytrack_fontsize=18,
                                                    height=0.35, start=0, end=len(ctg_1_record),
                                                    scale=True, scale_fontsize=12, scale_ticks=1,
                                                    scale_smalltick_interval=10000, scale_largetick_interval=10000)

    # add gene content track for gene2_contig
    ctg_2_name_str = '%s (%sbp)    Identity (%s-%s%s)    Alignment length (%s-%sbp)' % (ctg_2_record.id, len(ctg_2_record), iden_min, iden_max, '%', aln_len_min, aln_len_max)
    ctg_2_name_str = '%s (%sbp)' % (ctg_2_record.id, len(ctg_2_record))
    contig_2_gene_content_track = diagram.new_track(1, name=ctg_2_name_str, greytrack=True, greytrack_labels=1,
                                                    greytrack_font='Helvetica', greytrack_fontsize=18,
                                                    height=0.35, start=0, end=len(ctg_2_record),
                                                    scale=True, scale_fontsize=12, scale_ticks=1,
                                                    scale_smalltick_interval=10000, scale_largetick_interval=10000)

    ####################################### add crosslink from blast results #######################################

    # parse blast results
    for each_line in open(blast_result_txt_filtered):
        each_line_split = each_line.split('\t')
        query        = each_line_split[0]
        identity     = float(each_line_split[2])
        query_start  = int(each_line_split[6])
        query_end    = int(each_line_split[7])
        target_start = int(each_line_split[8])
        target_end   = int(each_line_split[9])

        # use color to reflect identity
        color = colors.linearlyInterpolatedColor(colors.white, colors.red, iden_min, iden_max, identity)

        # if query is contig_1
        if query == ctg_1_record.id:
            link = CrossLink((contig_1_gene_content_track, query_start, query_end),
                             (contig_2_gene_content_track, target_start, target_end),
                             color=color, border=color, flip=False)
            diagram.cross_track_links.append(link)

        # if query is contig_2
        elif query == ctg_2_record.id:
            link = CrossLink((contig_2_gene_content_track, query_start, query_end),
                             (contig_1_gene_content_track, target_start, target_end),
                             color=color, border=color, flip=False)
            diagram.cross_track_links.append(link)

    # Draw and Export
    diagram.draw(format="linear", orientation="landscape", pagesize=(50*cm, 25*cm), fragments=1, start=0, end=max_len)
    diagram.write(output_plot, plot_format)


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-p', required=True,                 help='output prefix')
    parser.add_argument('-1', required=True,                 help='sequence 1, in fasta format')
    parser.add_argument('-2', required=True,                 help='sequence 2, in fasta format')
    parser.add_argument('-i', required=True,  type=float,    help='identity cutoff, 0-100')
    parser.add_argument('-l', required=True,  type=int,      help='alignment length cutoff, in bp')
    parser.add_argument('-f', required=False, default='PDF', help='plot format, choose from PDF, SVG, EPS and PNG, default: PDF')
    args = vars(parser.parse_args())
    cross_link_seqs(args)



#!/usr/bin/env python

__author__ = "Xiang Gao, Huaiying Lin, Qunfeng Dong"
__copyright__ = "Copyright 2016, Bayesian-based LCA taxonomic classification"
__credits__ = ["Xiang Gao", "Huaiying Lin","Qunfeng Dong"]
__license__ = "GPL"
__version__ = "2.1"
__maintainer__ = "Huaiying Lin"
__email__ = "hlin2@luc.edu"
__status__ = "Python3"

import argparse
import os
import sys
import pandas as pd
import numpy as np
import time

#### constant #####
RANKNAMES = ["superkingdom","phylum","class","order","family","genus","species"]
headers = [(r,r+"-score") for r in RANKNAMES]
headers = [y for z in headers for y in z]

##### function #####
def float_100range(x):
    x = float(x)
    if x < 0.0 or x > 100.0:
        raise argparse.ArgumentTypeError("%r not in range [0.0, 100.0]"%(x,))
    return x

def blca_parser(blcaout, level, cutoff, rel):
    na = os.path.splitext(os.path.split(os.path.basename(blcaout))[1])[0]
    try:
        blca = pd.read_csv(blcaout, sep="\t", header=None)
        blca.columns = ["ReadID", "Taxonomy"]
        tx = blca["Taxonomy"].str.split(";|:", expand=True)
        tx.replace("", np.nan, inplace=True)
        tx = tx.dropna(how='all', axis=1)
    except:
        print("Your input file is not in BLCA output format!")
        sys.exit(1)
    tx = tx.drop([x for x in range(0,tx.shape[1],3)],axis=1)
    tx.columns = headers
    total = tx.shape[0]
    tx[[r+"-score" for r in RANKNAMES]] = tx[[r+"-score" for r in RANKNAMES]].astype(float)
    sel_tx = tx.loc[:,[s.startswith(level) for s in tx.columns]]
    sel_tx = sel_tx.loc[sel_tx[level+"-score"] > cutoff,:]
    outtx = sel_tx.groupby(level).agg("count")
    outtx.columns = [na]
    uclRow = pd.Series({na: total - int(outtx.sum(axis=0))}, name="Unclassified")
    outtx = outtx.append(uclRow)
    if rel:
        outtx[na] = outtx[na]/total * 100
    return outtx

def loop_wrapper(list_of_blcaouts,level, cutoff, rel):
    fF = list_of_blcaouts.pop(0)
    merDF = blca_parser(fF, level, cutoff, rel)
    for f in list_of_blcaouts:
        tmpDF = blca_parser(f, level, cutoff, rel)
        merDF = merDF.merge(tmpDF, on=level, how="outer").fillna(0)
    return merDF

def main():
    Nf = len(args.files)
    startTime = time.time()
    outFile = loop_wrapper(args.files, args.tax, args.cutoff, not args.rawCounts)
    outFile.to_csv(args.output,sep='\t')
    endTime = time.time()
    print("Merged %i files" % Nf)
    print("Time elapsed: %i seconds" % (round((endTime - startTime))))


##### parser ######

parser = argparse.ArgumentParser(prog = "generate_abundance_table.py", description=
     '''    Utility script to combine all BLCA outputs and generate abundance table.
    
    Sample ID will be automatically extracted from the file name as the ID used in the combined table.
    
    Multiple files or wild card file pattern are accepted.''',
     epilog="No warrenty comes with this script. Author: hlin2@luc.edu. \nAny suggestions or bugs report are welcomed.",
     add_help=False, formatter_class=argparse.RawTextHelpFormatter)
##### Required arguments #####
required = parser.add_argument_group('positional arguments')
required.add_argument("files", metavar = "input.blca.out", nargs = "+", help="Input BLCA output files.")
##### Reads filter #####
readsoptions = parser.add_argument_group('reads filter arguments')
readsoptions.add_argument("-c", "--cutoff", default=80.0, help="minimum confidence score to be considered as a valid taxonomy assignment. Default: 80", type=float_100range)
##### Other arguments #####
optional = parser.add_argument_group('other arguments')
optional.add_argument("-t", "--tax", default='genus',
                      help="taxonomy level to construct the table. Default: genus", type=str,
                      choices=["phylum","class","order","family","genus","species"])
optional.add_argument("--rawCounts", help="use raw counts instead of relative abundance",
                      action='store_true')
optional.add_argument("-o", "--output",
                      help="output file name. Default: merged.table.<level>.txt", type=str)
optional.add_argument("-h", "--help", help="show this help message and exit", action="help")
##### parse arguments #####
args = parser.parse_args()
##### Output file name ####
if not args.output:
    args.output="merged.table."+args.tax+".txt"

# print(args.files)
if __name__ == "__main__":
    main()


import os
import argparse


get_eu_otu_usage = '''
================================= get_eu_otu example commands =================================

BioSAK get_eu_otu -i otu_vs_nt.txt -o EU_OTU.txt -node nodes.dmp -db RefSeq_taxonomy.txt -n 10

# An OTU was considered as eukaryotic if more than half of its hits are from Eukaryotes.

===============================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def group_tax_by_domain(nodes_dmp):
    child_to_parent_dict = dict()
    for each_node in open(nodes_dmp):
        each_node_split = each_node.strip().split('|')
        tax_id = int(each_node_split[0].strip())
        parent_tax_id = int(each_node_split[1].strip())
        child_to_parent_dict[tax_id] = parent_tax_id

    tax_id_set_bac = set()
    tax_id_set_ar = set()
    tax_id_set_eu = set()
    tax_id_set_virus = set()
    for tax_id in child_to_parent_dict.keys():
        # get tax_lineage_list
        tax_lineage_list = [tax_id]
        while tax_id != 1:
            tax_id = child_to_parent_dict[tax_id]
            tax_lineage_list.append(tax_id)
        # Bacteria
        if 2 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_bac.add(each_id)
        # Archaea
        elif 2157 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_ar.add(each_id)
        # Eukaryota
        elif 2759 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_eu.add(each_id)
        # Virus
        elif 10239 in tax_lineage_list:
            for each_id in tax_lineage_list[:-3]:
                tax_id_set_virus.add(each_id)
    return tax_id_set_bac, tax_id_set_ar, tax_id_set_eu, tax_id_set_virus


def get_eu_otu(args):

    blast_op_txt            = args['i']
    eu_otu_txt              = args['o']
    nodes_dmp               = args['node']
    refseq_tax_txt          = args['db']
    hit_num_to_keep         = args['n']
    consider_as_eu_cutoff   = args['c']

    f_name, f_path, f_base, f_ext = sep_path_basename_ext(eu_otu_txt)
    cmd_txt = '%s/%s_RefSeq_classification.sh' % (f_path, f_base)
    tax_set_bac, tax_set_ar, tax_set_eu, tax_set_virus = group_tax_by_domain(nodes_dmp)

    refseq_tax_dict = dict()
    for each_ref in open(refseq_tax_txt):
        each_ref_split = each_ref.strip().split('\t')
        if len(each_ref_split) == 2:
            refseq_tax_dict[each_ref_split[0]] = each_ref_split[1]

    query_to_hit_dict = dict()
    for each_line in open(blast_op_txt):
        each_line_split = each_line.strip().split('\t')
        query_id = each_line_split[0]
        subject_id = each_line_split[1]
        if query_id not in query_to_hit_dict:
            query_to_hit_dict[query_id] = set()
        if len(query_to_hit_dict[query_id]) < hit_num_to_keep:
            query_to_hit_dict[query_id].add(subject_id)

    unclassified_refseq_set = set()
    eu_otu_set = set()
    for query in query_to_hit_dict:
        hit_tax_list = []
        for each_hit in query_to_hit_dict[query]:
            hit_tax = refseq_tax_dict.get(each_hit, 'na')
            if hit_tax == 'na':
                hit_tax_list.append(hit_tax)
                unclassified_refseq_set.add(each_hit)
            else:
                hit_tax_list.append(int(hit_tax))

        hit_num_bac = 0
        hit_num_ar = 0
        hit_num_eu = 0
        hit_num_virus = 0
        hit_num_na = 0
        for hit_tax in hit_tax_list:
            if hit_tax in tax_set_bac:
                hit_num_bac += 1
            elif hit_tax in tax_set_ar:
                hit_num_ar += 1
            elif hit_tax in tax_set_eu:
                hit_num_eu += 1
            elif hit_tax in tax_set_virus:
                hit_num_virus += 1
            elif hit_tax == 'na':
                hit_num_na += 1

        hit_num_without_na = hit_num_bac + hit_num_ar + hit_num_eu + hit_num_virus

        # An OTU was considered as eukaryotic if more than half of the hits are from Eukaryotes
        eu_hit_pct = 0
        if hit_num_without_na > 0:
            eu_hit_pct = hit_num_eu / hit_num_without_na
        if eu_hit_pct > consider_as_eu_cutoff:
            eu_otu_set.add(query)

    # write out EU OTU
    eu_otu_txt_handle = open(eu_otu_txt, 'w')
    eu_otu_txt_handle.write('\n'.join(sorted(list(eu_otu_set))))
    eu_otu_txt_handle.close()

    # report
    print('Note:')
    if len(unclassified_refseq_set) > 0:
        cmd_txt_handle = open(cmd_txt, 'w')
        for seq_accession in unclassified_refseq_set:
            cmd = 'esearch -db nucleotide -query %s | efetch -format docsum | xtract -pattern DocumentSummary -element TaxId > %s.txt' % (seq_accession, seq_accession)
            cmd_txt_handle.write(cmd + '\n')
        cmd_txt_handle.close()
        print('Taxon ID for %s RefSeqs is missing, to increase accuracy, add their classifications to %s and rerun this script.' % (len(unclassified_refseq_set), refseq_tax_txt))
        print('RefSeq classification commands exported to: %s' % cmd_txt)
    print('The ID of %s eukaryotic OTUs has exported to: %s' % (len(eu_otu_set), eu_otu_txt))
    print('Done!')


if __name__ == '__main__':

    get_eu_otu_parser = argparse.ArgumentParser(usage=get_eu_otu_usage)
    get_eu_otu_parser.add_argument('-i',    required=True,                              help='blast result, outfmt need to be 6')
    get_eu_otu_parser.add_argument('-o',    required=True,                              help='output of identified eukaryotic OTUs')
    get_eu_otu_parser.add_argument('-node', required=True,                              help='nodes.dmp ')
    get_eu_otu_parser.add_argument('-db',   required=True,                              help='RefSeq taxonomy file')
    get_eu_otu_parser.add_argument('-c',    required=False, type=float, default=0.3,    help='consider as eukaryotic if the proportion of eukaryotic hits higher than this cutoff, default is 0.3')
    get_eu_otu_parser.add_argument('-n',    required=False, type=int, default=10,       help='number of top hits to consider, default is 10')
    args = vars(get_eu_otu_parser.parse_args())
    get_eu_otu(args)



import os
import argparse


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


compare_usage = '''
========= compare example command =========

BioSAK compare -1 file1.txt -2 file2.txt

===========================================
'''


def compare(args):

    file_1 = args['1']
    file_2 = args['2']

    _, _, f1_base, _ = sep_path_basename_ext(file_1)
    _, _, f2_base, _ = sep_path_basename_ext(file_2)

    file_1_uniq    = '%s_uniq.txt'      % f1_base
    file_2_uniq    = '%s_uniq.txt'      % f2_base
    file_12_shared = '%s_%s_shared.txt' % (f1_base, f2_base)

    set_1 = set()
    for each_line in open(file_1):
        set_1.add(each_line.strip().split()[0])

    set_2 = set()
    for each_line in open(file_2):
        set_2.add(each_line.strip().split()[0])

    shared_id_set = set(set_1).intersection(set_2)

    set_1_uniq = set()
    for e1 in set_1:
        if e1 not in shared_id_set:
            set_1_uniq.add(e1)

    set_2_uniq = set()
    for e2 in set_2:
        if e2 not in shared_id_set:
            set_2_uniq.add(e2)

    if len(shared_id_set) == 0:
        print('No id shared between %s and %s' % (file_1, file_2))
    else:
        if len(set_1_uniq) > 0:
            file_1_uniq_handle = open(file_1_uniq, 'w')
            file_1_uniq_handle.write('\n'.join(sorted(list(set_1_uniq))) + '\n')
            file_1_uniq_handle.close()
        if len(set_2_uniq) > 0:
            file_2_uniq_handle = open(file_2_uniq, 'w')
            file_2_uniq_handle.write('\n'.join(sorted(list(set_2_uniq))) + '\n')
            file_2_uniq_handle.close()
        if len(shared_id_set) > 0:
            file_12_shared_uniq_handle = open(file_12_shared, 'w')
            file_12_shared_uniq_handle.write('\n'.join(sorted(list(shared_id_set))) + '\n')
            file_12_shared_uniq_handle.close()


if __name__ == '__main__':

    compare_parser = argparse.ArgumentParser(usage=compare_usage)
    compare_parser.add_argument('-1', required=True, help='file 1')
    compare_parser.add_argument('-2', required=True, help='file 2')
    args = vars(compare_parser.parse_args())
    compare(args)



import argparse

taxdump_usage = '''
====================== taxdump example commands ======================

BioSAK taxdump -node nodes.dmp -name names.dmp -o ncbi_taxonomy.txt

# Input files available at:
https://ftp.ncbi.nih.gov/pub/taxonomy/taxdump.tar.gz

======================================================================
'''


def taxdump(args):

    nodes_dmp   = args['node']
    names_dmp   = args['name']
    op_txt      = args['o']

    tax_rank_abbrev_dict = {'domain' : 'd',
                            'phylum' : 'p', 'subphylum' : 'sp', 'superphylum' : 'up',
                            'class'  : 'c', 'subclass'  : 'sc', 'superclass'  : 'uc',
                            'order'  : 'o', 'suborder'  : 'so', 'superorder'  : 'uo',
                            'family' : 'f', 'subfamily' : 'sf', 'superfamily' : 'uf',
                            'genus'  : 'g', 'subgenus'  : 'sg', 'supergenus'  : 'ug',
                            'species': 's', 'subspecies': 'ss', 'superspecies': 'us',
                            'kingdom': 'k', 'subkingdom': 'sk', 'superkingdom': 'uk'}

    id_to_name_dict = dict()
    for each_node in open(names_dmp):
        each_node_split = each_node.strip().split('|')
        node_id    = int(each_node_split[0].strip())
        node_name  = each_node_split[1].strip()
        name_class = each_node_split[3].strip()
        if name_class == 'scientific name':
            id_to_name_dict[node_id] = node_name

    tax_set = set()
    tax_rank_dict = dict()
    child_to_parent_dict = dict()
    for each_node in open(nodes_dmp):
        each_node_split = each_node.strip().split('|')
        tax_id = int(each_node_split[0].strip())
        parent_tax_id = int(each_node_split[1].strip())
        tax_rank = each_node_split[2].strip()
        child_to_parent_dict[tax_id] = parent_tax_id
        tax_rank_dict[tax_id] = tax_rank
        if tax_rank not in ['species', 'species group', 'subspecies', 'no rank', 'strain']:
            tax_set.add(tax_id)

    op_txt_handle = open(op_txt, 'w')
    for tax_id in sorted(list(tax_set)):
        current_tax_name = id_to_name_dict[tax_id]
        # get tax_lineage_list
        tax_lineage_list = [tax_id]
        while tax_id != 1:
            tax_id = child_to_parent_dict[tax_id]
            tax_lineage_list.append(tax_id)

        # get tax_lineage_list_by_name
        tax_lineage_list_by_name = []
        for each_id in tax_lineage_list:
            tax_name = id_to_name_dict[each_id]
            tax_rank = tax_rank_dict[each_id]
            tax_rank_abbrev = tax_rank_abbrev_dict.get(tax_rank, tax_rank)
            if tax_rank in tax_rank_abbrev_dict:
                tax_lineage_list_by_name.append('%s__%s' % (tax_rank_abbrev, tax_name))

        tax_lineage_str = ';'.join(tax_lineage_list_by_name[::-1])

        # add g__ if missing
        if 'g__' not in tax_lineage_str:
            tax_lineage_str = tax_lineage_str + ';g__'

        # add f__ if missing
        if 'f__' not in tax_lineage_str:
            if 'ug__' in tax_lineage_str:
                tax_lineage_str = tax_lineage_str.replace(';ug__', ';f__;ug__')
            else:
                tax_lineage_str = tax_lineage_str.replace(';g__', ';f__;g__')

        # add o__ if missing
        if 'o__' not in tax_lineage_str:
            if 'uf__' in tax_lineage_str:
                tax_lineage_str = tax_lineage_str.replace(';uf__', ';o__;uf__')
            else:
                tax_lineage_str = tax_lineage_str.replace(';f__', ';o__;f__')

        # add c__ if missing
        if 'c__' not in tax_lineage_str:
            if 'uo__' in tax_lineage_str:
                tax_lineage_str = tax_lineage_str.replace(';uo__', ';c__;uo__')
            else:
                tax_lineage_str = tax_lineage_str.replace(';o__', ';c__;o__')

        # add p__ if missing
        if 'p__' not in tax_lineage_str:
            if 'uc__' in tax_lineage_str:
                tax_lineage_str = tax_lineage_str.replace(';uc__', ';p__;uc__')
            else:
                tax_lineage_str = tax_lineage_str.replace(';c__', ';p__;c__')

        # write out
        op_txt_handle.write('%s\t%s\n' % (current_tax_name, tax_lineage_str))
    op_txt_handle.close()


if __name__ == '__main__':

    taxdump_parser = argparse.ArgumentParser(usage=taxdump_usage)
    taxdump_parser.add_argument('-node',  required=True,    help='nodes.dmp')
    taxdump_parser.add_argument('-name',  required=True,    help='names.dmp')
    taxdump_parser.add_argument('-o',     required=True,    help='output file')
    args = vars(taxdump_parser.parse_args())
    taxdump(args)
